import time
import hashlib
import uuid
import io
from pathlib import Path


def uuid_from_stream(stream, *, init_string=""):
    """
    :return: 16-byte digest of stream data
    :stream: stream object or open file handle
    :init_string: string to initialize the checksum
    """
    hashed = hashlib.md5(init_string.encode())
    chunk = True
    chunk_size = 1 << 14
    while chunk:
        chunk = stream.read(chunk_size)
        hashed.update(chunk)
    return uuid.UUID(bytes=hashed.digest())


def uuid_from_buffer(buffer=b"", *, init_string=""):
    return uuid_from_stream(io.BytesIO(buffer), init_string=init_string)


def uuid_from_file(filepath, *, init_string=""):
    return uuid_from_stream(Path(filepath).open("rb"), init_string=init_string)



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# MAIN FILE
# ///////////////////////////////////////////////////////////////
from djsciops.axon_gui.main import *
from pathlib import Path
import djsciops

# GLOBALS
# ///////////////////////////////////////////////////////////////
GLOBAL_STATE = False
GLOBAL_TITLE_BAR = True


class UIFunctions(MainWindow):
    # MAXIMIZE/RESTORE
    # ///////////////////////////////////////////////////////////////
    def maximize_restore(self):
        global GLOBAL_STATE
        status = GLOBAL_STATE
        if status == False:
            self.showMaximized()
            GLOBAL_STATE = True
            self.ui.appMargins.setContentsMargins(0, 0, 0, 0)
            self.ui.maximizeRestoreAppBtn.setToolTip("Restore")
            self.ui.maximizeRestoreAppBtn.setIcon(
                QIcon(":/icons/images/icons/icon_restore.png")
            )
            self.ui.frame_size_grip.hide()
            self.left_grip.hide()
            self.right_grip.hide()
            self.top_grip.hide()
            self.bottom_grip.hide()
        else:
            GLOBAL_STATE = False
            self.showNormal()
            self.resize(self.width() + 1, self.height() + 1)
            self.ui.appMargins.setContentsMargins(10, 10, 10, 10)
            self.ui.maximizeRestoreAppBtn.setToolTip("Maximize")
            self.ui.maximizeRestoreAppBtn.setIcon(
                QIcon(":/icons/images/icons/icon_maximize.png")
            )
            self.ui.frame_size_grip.show()
            self.left_grip.show()
            self.right_grip.show()
            self.top_grip.show()
            self.bottom_grip.show()

    # RETURN STATUS
    # ///////////////////////////////////////////////////////////////
    def returStatus(self):
        return GLOBAL_STATE

    # SET STATUS
    # ///////////////////////////////////////////////////////////////
    def setStatus(self, status):
        global GLOBAL_STATE
        GLOBAL_STATE = status

    # TOGGLE MENU
    # ///////////////////////////////////////////////////////////////
    def toggleMenu(self, enable):
        if enable:
            # GET WIDTH
            width = self.ui.leftMenuBg.width()
            maxExtend = Settings.MENU_WIDTH
            standard = 60

            # SET MAX WIDTH
            if width == 60:
                widthExtended = maxExtend
            else:
                widthExtended = standard

            # ANIMATION
            self.animation = QPropertyAnimation(self.ui.leftMenuBg, b"minimumWidth")
            self.animation.setDuration(Settings.TIME_ANIMATION)
            self.animation.setStartValue(width)
            self.animation.setEndValue(widthExtended)
            self.animation.setEasingCurve(QEasingCurve.InOutQuart)
            self.animation.start()

    # TOGGLE LEFT BOX
    # ///////////////////////////////////////////////////////////////
    def toggleLeftBox(self, enable):
        if enable:
            # GET WIDTH
            width = self.ui.extraLeftBox.width()
            maxExtend = Settings.LEFT_BOX_WIDTH
            color = Settings.BTN_LEFT_BOX_COLOR
            standard = 0

            # GET BTN STYLE
            style = self.ui.toggleLeftBox.styleSheet()

            # SET MAX WIDTH
            if width == 0:
                widthExtended = maxExtend
                # SELECT BTN
                self.ui.toggleLeftBox.setStyleSheet(style + color)
            else:
                widthExtended = standard
                # RESET BTN
                self.ui.toggleLeftBox.setStyleSheet(style.replace(color, ""))

        UIFunctions.start_config_box_animation(self, width, "left")

    def start_config_box_animation(self, left_box_width, direction):
        left_width = 0

        # Check values
        if left_box_width == 0 and direction == "left":
            left_width = Settings.LEFT_BOX_WIDTH
        else:
            left_width = 0

        # ANIMATION LEFT BOX
        self.left_box = QPropertyAnimation(self.ui.extraLeftBox, b"minimumWidth")
        self.left_box.setDuration(Settings.TIME_ANIMATION)
        self.left_box.setStartValue(left_box_width)
        self.left_box.setEndValue(left_width)
        self.left_box.setEasingCurve(QEasingCurve.InOutQuart)

        # GROUP ANIMATION
        self.group = QParallelAnimationGroup()
        self.group.addAnimation(self.left_box)
        self.group.start()

    def toggleConsole(self):
        # GET HEIGHT
        height = self.ui.bottomBar.height()
        maxExtend = Settings.CONSOLE_BOX_HEIGHT
        standard = 22

        # SET MAX WIDTH
        if height == 22:
            heightExtended = maxExtend
            self.ui.toggleConsoleBtn.setIcon(
                QIcon(":/icons/images/icons/cil-caret-bottom.png")
            )
        else:
            heightExtended = standard
            self.ui.toggleConsoleBtn.setIcon(
                QPixmap(":icons/images/icons/cil-caret-right.png")
            )

        UIFunctions.start_console_box_animation(self, height)
        self.ui.upload_status.verticalScrollBar().setValue(
            self.ui.upload_status.verticalScrollBar().maximum()
        )
        self.ui.upload_status.ensureCursorVisible()

    def start_console_box_animation(self, console_box_height):
        console_height = 22

        # Check values
        if console_box_height == 22:
            console_height = Settings.CONSOLE_BOX_HEIGHT
        else:
            console_height = 22

        # ANIMATION LEFT BOX
        self.console_box = QPropertyAnimation(self.ui.bottomBar, b"minimumHeight")
        self.console_box.setDuration(Settings.TIME_ANIMATION)
        self.console_box.setStartValue(console_box_height)
        self.console_box.setEndValue(console_height)
        self.console_box.setEasingCurve(QEasingCurve.InOutQuart)

        # GROUP ANIMATION
        self.group = QParallelAnimationGroup()
        self.group.addAnimation(self.console_box)
        self.group.start()

    # SELECT/DESELECT MENU
    # ///////////////////////////////////////////////////////////////
    # SELECT
    def selectMenu(getStyle):
        select = getStyle + Settings.MENU_SELECTED_STYLESHEET
        return select

    # DESELECT
    def deselectMenu(getStyle):
        deselect = getStyle.replace(Settings.MENU_SELECTED_STYLESHEET, "")
        return deselect

    # RESET SELECTION
    def resetStyle(self, widget):
        for w in self.ui.topMenu.findChildren(QPushButton):
            if w.objectName() != widget:
                w.setStyleSheet(UIFunctions.deselectMenu(w.styleSheet()))

    # IMPORT THEMES FILES QSS/CSS
    # ///////////////////////////////////////////////////////////////
    def theme(self, file, useCustomTheme):
        if useCustomTheme:
            str = open(file, "r").read()
            self.ui.styleSheet.setStyleSheet(str)

    # START - GUI DEFINITIONS
    # ///////////////////////////////////////////////////////////////
    def uiDefinitions(self):
        def dobleClickMaximizeRestore(event):
            # IF DOUBLE CLICK CHANGE STATUS
            if event.type() == QEvent.MouseButtonDblClick:
                QTimer.singleShot(250, lambda: UIFunctions.maximize_restore(self))

        self.ui.titleRightInfo.mouseDoubleClickEvent = dobleClickMaximizeRestore

        if Settings.ENABLE_CUSTOM_TITLE_BAR:
            # STANDARD TITLE BAR
            self.setWindowFlags(Qt.FramelessWindowHint)
            self.setAttribute(Qt.WA_TranslucentBackground)

            # MOVE WINDOW / MAXIMIZE / RESTORE
            def moveWindow(event):
                # IF MAXIMIZED CHANGE TO NORMAL
                if UIFunctions.returStatus(self):
                    UIFunctions.maximize_restore(self)
                # MOVE WINDOW
                if event.buttons() == Qt.LeftButton:
                    self.move(self.pos() + event.globalPos() - self.dragPos)
                    self.dragPos = event.globalPos()
                    event.accept()

            self.ui.titleRightInfo.mouseMoveEvent = moveWindow

            # CUSTOM GRIPS
            self.left_grip = CustomGrip(self, Qt.LeftEdge, True)
            self.right_grip = CustomGrip(self, Qt.RightEdge, True)
            self.top_grip = CustomGrip(self, Qt.TopEdge, True)
            self.bottom_grip = CustomGrip(self, Qt.BottomEdge, True)

        else:
            self.ui.appMargins.setContentsMargins(0, 0, 0, 0)
            self.ui.minimizeAppBtn.hide()
            self.ui.maximizeRestoreAppBtn.hide()
            self.ui.closeAppBtn.hide()
            self.ui.frame_size_grip.hide()

        # DROP SHADOW
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setBlurRadius(17)
        self.shadow.setXOffset(0)
        self.shadow.setYOffset(0)
        self.shadow.setColor(QColor(0, 0, 0, 150))
        self.ui.bgApp.setGraphicsEffect(self.shadow)

        # RESIZE WINDOW
        self.sizegrip = QSizeGrip(self.ui.frame_size_grip)
        self.sizegrip.setStyleSheet(
            "width: 20px; height: 20px; margin 0px; padding: 0px;"
        )

        # MINIMIZE
        self.ui.minimizeAppBtn.clicked.connect(lambda: self.showMinimized())

        # MAXIMIZE/RESTORE
        self.ui.maximizeRestoreAppBtn.clicked.connect(
            lambda: UIFunctions.maximize_restore(self)
        )

        # CLOSE APPLICATION
        self.ui.closeAppBtn.clicked.connect(lambda: self.close())

    def resize_grips(self):
        if Settings.ENABLE_CUSTOM_TITLE_BAR:
            self.left_grip.setGeometry(0, 10, 10, self.height())
            self.right_grip.setGeometry(self.width() - 10, 10, 10, self.height())
            self.top_grip.setGeometry(0, 0, self.width(), 10)
            self.bottom_grip.setGeometry(0, self.height() - 10, self.width(), 10)

    # ///////////////////////////////////////////////////////////////
    # END - GUI DEFINITIONS

    # DIRECTORY TREE
    # ///////////////////////////////////////////////////////////////
    def refresh_tree(tree, path_or_dict, method):
        tree.clear()
        if method == "dict":
            UIFunctions.fill_tree_from_dict(tree.invisibleRootItem(), path_or_dict)
        elif method == "path":
            UIFunctions.fill_tree_from_path(
                tree.invisibleRootItem(), path_or_dict, originalpath=path_or_dict
            )
        tree.sortItems(0, Qt.AscendingOrder)

    def fill_tree_from_dict(item, value):
        def new_item(parent, text, val=None):
            child = QTreeWidgetItem([text])
            if len(text.split(".")) == 1:
                child.setIcon(0, QIcon(":/icons/images/icons/cil-folder.png"))
            else:
                child.setIcon(0, QIcon(":/icons/images/icons/cil-file.png"))
            UIFunctions.fill_tree_from_dict(child, val)
            parent.addChild(child)
            child.setExpanded(True)

        if value is None:
            return
        elif isinstance(value, dict):
            for key, val in sorted(value.items()):
                new_item(item, str(key), val)

    def fill_tree_from_path(
        tree, startpath, originalpath=None, checkable=True, checklist=None
    ):
        def new_item(parent, text, path):
            child = QTreeWidgetItem([text, path])
            if os.path.isdir(path):
                child.setIcon(0, QIcon(":/icons/images/icons/cil-folder.png"))
                child.setFlags(
                    child.flags() | Qt.ItemIsAutoTristate | Qt.ItemIsUserCheckable
                )
            else:
                child.setIcon(0, QIcon(":/icons/images/icons/cil-file.png"))
                child.setFlags(child.flags() | Qt.ItemIsUserCheckable)
            if checkable:
                UIFunctions.fill_tree_from_path(child, path)
                child.setCheckState(0, Qt.Checked)
                parent.addChild(child)
                child.setExpanded(True)
            else:
                if text in checklist:
                    UIFunctions.fill_tree_from_path(
                        child, path, checkable=checkable, checklist=checklist
                    )
                    parent.addChild(child)

        if os.path.isfile(startpath):
            return
        else:
            if startpath == originalpath:
                new_item(tree, os.path.basename(startpath), startpath)
            else:
                for element in os.listdir(startpath):
                    path_info = startpath + "/" + element
                    new_item(tree, os.path.basename(element), path_info)

    def update_bucket_tree(self):
        try:
            UIFunctions.refresh_tree(
                self.ui.browse_s3_bucket,
                djsciops_axon.list_files(
                    session=self.session,
                    s3_bucket=self.ui.s3_bucket.text(),
                    s3_prefix=self.ui.browse_s3_dir.text(),
                ),
                "dict",
            )
            log.info("Successfully Loaded Bucket")
        except Exception as e:
            log.error(f"Failed to load Bucket: {e}")

    def update_bucket_preview(self):
        self.ui.s3_bucket_dir.clear()
        self.ui.s3_dir.setText(
            self.ui.s3_dir.text() + "/"
            if self.ui.s3_dir.text()[-1] != "/"
            else self.ui.s3_dir.text()
        )
        child = QTreeWidgetItem([self.ui.s3_dir.text()])
        child.setIcon(0, QIcon(":/icons/images/icons/cil-folder.png"))

        checklist = []
        iterator = QTreeWidgetItemIterator(
            self.ui.local_dir_tree, QTreeWidgetItemIterator.Checked
        )
        while iterator.value():
            item = iterator.value()
            checklist.append(item.text(0))
            iterator += 1
        UIFunctions.fill_tree_from_path(
            child, self.ui.local_dir.text(), checkable=False, checklist=checklist
        )
        self.ui.s3_bucket_dir.invisibleRootItem().addChild(child)
        child.setExpanded(True)
        self.ui.s3_bucket_dir.sortItems(0, Qt.AscendingOrder)



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

import sys
import os
import platform
import traceback
import logging
from datetime import datetime
from pathlib import Path, PurePosixPath

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////
from djsciops import __version__ as axon_version
from djsciops import axon as djsciops_axon
from djsciops import authentication as djsciops_authentication
from djsciops import settings as djsciops_settings
from djsciops import utils as djsciops_utils
from djsciops.log import log, log_format
import djsciops

from djsciops.axon_gui.modules import *
from djsciops.axon_gui.widgets import *

os.environ["QT_FONT_DPI"] = "96"  # FIX Problem for High DPI and Scale above 100%
basedir = os.path.dirname(__file__)

# SET AS GLOBAL WIDGETS
# ///////////////////////////////////////////////////////////////
widgets = None


class WorkerSignals(QObject):
    """
    Defines the signals available from a running worker thread.

    Supported signals are:

    finished
        No data

    error
        tuple (exctype, value, traceback.format_exc() )

    result
        object data returned from processing, anything

    """

    finished = Signal()
    error = Signal(tuple)
    result = Signal(object)


class Worker(QRunnable):
    """
    Worker thread

    Inherits from QRunnable to handler worker thread setup, signals and wrap-up.

    :param callback: The function callback to run on this worker thread. Supplied args and
                     kwargs will be passed through to the runner.
    :type callback: function
    :param args: Arguments to pass to the callback function
    :param kwargs: Keywords to pass to the callback function

    """

    def __init__(self, fn, *args, **kwargs):
        super(Worker, self).__init__()

        # Store constructor arguments (re-used for processing)
        self.fn = fn
        self.args = args
        self.kwargs = kwargs
        self.signals = WorkerSignals()

    @Slot()
    def run(self):
        """
        Initialise the runner function with passed args, kwargs.
        """

        # Retrieve args/kwargs here; and fire processing using them
        try:
            result = self.fn(*self.args, **self.kwargs)
        except:
            traceback.print_exc()
            exctype, value = sys.exc_info()[:2]
            self.signals.error.emit((exctype, value, traceback.format_exc()))
        else:
            self.signals.result.emit(result)  # Return the result of the processing
        finally:
            self.signals.finished.emit()  # Done


class MainWindow(QMainWindow):
    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)

        # SET AS GLOBAL WIDGETS
        # ///////////////////////////////////////////////////////////////
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        global widgets
        widgets = self.ui

        # DEFINE LOG HANDLER
        # ///////////////////////////////////////////////////////////////
        class GuiLogHandler(logging.Handler):
            def emit(self, log_record):
                widgets.upload_status.append(self.format(log_record))
                widgets.upload_status.verticalScrollBar().setValue(widgets.upload_status.verticalScrollBar().maximum())
                widgets.upload_status.ensureCursorVisible()

        gui_handler = GuiLogHandler()
        gui_handler.setFormatter(
            logging.Formatter("[%(asctime)s][%(levelname)s]: %(message)s", "%H:%M:%S")
        )
        log.addHandler(gui_handler)

        # USE CUSTOM TITLE BAR | USE AS "False" FOR MAC OR LINUX
        # ///////////////////////////////////////////////////////////////
        Settings.ENABLE_CUSTOM_TITLE_BAR = True

        # APP NAME
        # ///////////////////////////////////////////////////////////////
        # APPLY TEXTS
        self.setWindowTitle("Axon")
        widgets.titleRightInfo.setText("Axon Upload")

        # Session Token
        # ///////////////////////////////////////////////////////////////
        self.session = None

        # TOGGLE BUTTONS
        # ///////////////////////////////////////////////////////////////
        # tab expansion
        widgets.toggleButton.clicked.connect(lambda: UIFunctions.toggleMenu(self, True))
        # config menu
        widgets.toggleLeftBox.clicked.connect(
            lambda: UIFunctions.toggleLeftBox(self, True)
        )
        widgets.extraCloseColumnBtn.clicked.connect(
            lambda: UIFunctions.toggleLeftBox(self, True)
        )
        # switch theme
        widgets.themeSwitchBtn.clicked.connect(lambda: self.switch_themes())
        # console
        widgets.toggleConsoleBtn.clicked.connect(
            lambda: UIFunctions.toggleConsole(self)
        )

        # SET UI DEFINITIONS
        # ///////////////////////////////////////////////////////////////
        UIFunctions.uiDefinitions(self)

        # SET CONFIG
        # ///////////////////////////////////////////////////////////////
        try:
            self.config = djsciops_settings.get_config(stdin_enabled=False)
        except FileNotFoundError:
            self.config = dict(
                version=axon_version,
                aws=dict(account_id=""),
                s3=dict(role="", bucket=""),
                djauth=dict(client_id=""),
            )
            widgets.upload_status.setText(
                "-> Set up your djsciops config to begin uploading"
            )

        # BUTTONS CLICK
        # ///////////////////////////////////////////////////////////////
        widgets.btn_upload.clicked.connect(self.buttonClick)
        widgets.btn_bucket.clicked.connect(self.buttonClick)
        widgets.btn_browse_dir.clicked.connect(self.buttonClick)
        widgets.btn_upload_files.clicked.connect(self.buttonClick)
        widgets.btn_save_config.clicked.connect(self.buttonClick)
        widgets.refreshTokenBtn.clicked.connect(self.generate_auth_token)

        # SHOW APP
        # ///////////////////////////////////////////////////////////////
        self.show()

        # SET THREADPOOL
        # ///////////////////////////////////////////////////////////////
        self.threadpool = QThreadPool()
        print(
            "Multithreading with maximum %d threads" % self.threadpool.maxThreadCount()
        )

        # SET CUSTOM THEME
        # ///////////////////////////////////////////////////////////////
        useCustomTheme = True
        self.theme = "dark"  # Default theme

        # SET THEME AND HACKS
        if useCustomTheme:
            widgets.themeSwitchBtn.setVisible(True)
            # LOAD AND APPLY STYLE
            UIFunctions.theme(
                self,
                os.path.join(basedir, "themes", f"py_dracula_{self.theme}.qss"),
                True,
            )

            # SET HACKS
            AppFunctions.setThemeHack(self)
        else:
            widgets.themeSwitchBtn.setVisible(False)

        # SET HOME PAGE AND SELECT MENU
        # ///////////////////////////////////////////////////////////////
        widgets.stackedWidget.setCurrentWidget(widgets.pageUpload)
        widgets.btn_upload.setStyleSheet(
            UIFunctions.selectMenu(widgets.btn_upload.styleSheet())
        )

        # FETCH CONFIG
        # ///////////////////////////////////////////////////////////////
        widgets.aws_acc_id.setText(self.config["aws"]["account_id"])
        widgets.s3_role.setText(self.config["s3"]["role"])
        widgets.s3_bucket.setText(self.config["s3"]["bucket"])
        widgets.dj_acc_client_id.setText(self.config["djauth"]["client_id"])

        # LOAD DIR TREE
        # ///////////////////////////////////////////////////////////////
        widgets.browse_s3_dir.returnPressed.connect(self.load_bucket_dir)
        widgets.btn_refresh_browser.clicked.connect(self.load_bucket_dir)
        widgets.s3_dir.returnPressed.connect(self.load_bucket_preview)
        widgets.btn_refresh_preview.clicked.connect(self.load_bucket_preview)
        widgets.local_dir_tree.itemClicked.connect(self.load_bucket_preview)

    # BUTTONS CLICK
    # Post here your functions for clicked buttons
    # ///////////////////////////////////////////////////////////////
    def buttonClick(self):
        # GET BUTTON CLICKED
        btn = self.sender()
        btnName = btn.objectName()

        # SHOW UPLOAD PAGE
        if btnName == "btn_upload":
            widgets.titleRightInfo.setText("Axon Upload")
            widgets.stackedWidget.setCurrentWidget(widgets.pageUpload)
            UIFunctions.resetStyle(self, btnName)
            btn.setStyleSheet(UIFunctions.selectMenu(btn.styleSheet()))

        # SHOW BROWSE BUCKET PAGE
        elif btnName == "btn_bucket":
            widgets.titleRightInfo.setText("S3 Bucket Browser")
            widgets.stackedWidget.setCurrentWidget(widgets.pageS3Browse)
            UIFunctions.resetStyle(self, btnName)
            btn.setStyleSheet(UIFunctions.selectMenu(btn.styleSheet()))

        # BROWSE LOCAL DIRECTORIES
        elif btnName == "btn_browse_dir":
            widgets.local_dir.setText(
                str(QFileDialog.getExistingDirectory(self, "Select Directory"))
            )
            if widgets.local_dir.text():
                try:
                    UIFunctions.refresh_tree(
                        widgets.local_dir_tree, widgets.local_dir.text(), "path"
                    )
                except Exception as e:
                    log.error(f"Failed to load directory: {e}")
                if widgets.s3_dir.text():
                    self.load_bucket_preview()
            else:
                widgets.local_dir_tree.clear()
                widgets.s3_bucket_dir.clear()

        # UPLOAD FILES
        elif btnName == "btn_upload_files":
            if not widgets.local_dir.text() or not widgets.s3_dir.text():
                log.warning("Select a source and destination to begin uploading")
            elif (
                not widgets.aws_acc_id.text()
                or not widgets.dj_acc_client_id.text()
                or not widgets.s3_role.text()
                or not widgets.s3_bucket.text()
            ):
                log.warning("Set up your djsciops config to begin uploading")
            elif self.check_auth_token():
                worker = Worker(self.upload_files)
                worker.signals.finished.connect(
                    lambda: widgets.btn_upload_files.setEnabled(True)
                )
                self.threadpool.start(worker)

        # SAVE CONFIG
        elif btnName == "btn_save_config":
            self.config["aws"]["account_id"] = widgets.aws_acc_id.text()
            self.config["djauth"]["client_id"] = widgets.dj_acc_client_id.text()
            self.config["s3"]["role"] = widgets.s3_role.text()
            self.config["s3"]["bucket"] = widgets.s3_bucket.text()
            try:
                config_directory = djsciops_settings.appdirs.user_data_dir(
                    appauthor="datajoint", appname="djsciops"
                )
                os.makedirs(config_directory, exist_ok=True)
                with open(
                    djsciops_settings.pathlib.Path(config_directory, "config.yaml"), "w"
                ) as f:
                    f.write(str(self.config))
                log.info("Config has been saved")
            except Exception as e:
                log.error(f"Failed to save config: {e}")

        # PRINT BTN NAME
        print(f'Button "{btnName}" pressed!')

    # WORKER FUNCTIONS
    # ///////////////////////////////////////////////////////////////
    def upload_files(self):
        try:
            widgets.btn_upload_files.setEnabled(False)
            checked_items, dest = self.map_checked_items(widgets.local_dir_tree)
            djsciops_axon.upload_files(
                session=self.session,
                s3_bucket=self.config["s3"]["bucket"],
                source=checked_items,
                destination=dest,
            )
            log.info("Upload complete")
        except Exception as e:
            log.error(f"Upload failed: {e}")

    def load_bucket_dir(self):
        if self.check_auth_token():
            UIFunctions.update_bucket_tree(self)

    def generate_auth_token(self):
        widgets.refreshTokenBtn.setEnabled(False)
        worker = Worker(self.set_session)
        worker.signals.finished.connect(self.finish_auth)
        self.threadpool.start(worker)

    # UTIL FUNCTIONS
    # ///////////////////////////////////////////////////////////////
    def load_bucket_preview(self):
        if widgets.local_dir.text() and widgets.s3_dir.text():
            try:
                UIFunctions.update_bucket_preview(self)
            except Exception as e:
                log.error(f"Failed to load bucket preview: {e}")

    def map_checked_items(self, tree):
        items = []
        dest = []
        widgets.local_dir.setText(
            widgets.local_dir.text() + "/"
            if widgets.local_dir.text()[-1] != "/"
            else widgets.local_dir.text()
        )
        widgets.s3_dir.setText(
            widgets.s3_dir.text() + "/"
            if widgets.s3_dir.text()[-1] != "/"
            else widgets.s3_dir.text()
        )
        iterator = QTreeWidgetItemIterator(tree, QTreeWidgetItemIterator.Checked)
        while iterator.value():
            item = iterator.value()
            if os.path.isfile(item.text(1)):
                items.append(Path(item.text(1)))
                dest.append(
                    PurePosixPath(
                        item.text(1).replace(
                            widgets.local_dir.text(), widgets.s3_dir.text()
                        )
                    )
                )
            iterator += 1
        return items, dest

    def switch_themes(self):
        if self.theme == "light":
            self.theme = "dark"
            # LOAD AND APPLY STYLE
            UIFunctions.theme(
                self,
                os.path.join(basedir, "themes", f"py_dracula_{self.theme}.qss"),
                True,
            )
            widgets.themeSwitchBtn.setIcon(QIcon(":/icons/images/icons/cil-sun.png"))

        elif self.theme == "dark":
            self.theme = "light"
            # LOAD AND APPLY STYLE
            UIFunctions.theme(
                self,
                os.path.join(basedir, "themes", f"py_dracula_{self.theme}.qss"),
                True,
            )

            widgets.themeSwitchBtn.setIcon(QIcon(":/icons/images/icons/cil-moon.png"))

    def check_auth_token(self):
        if not self.session:
            log.warning("Generate a new authentication token to begin using Axon")
            return False
        else:
            time_to_live = (
                (self.session.jwt["exp"] - datetime.now().timestamp()) / 60 / 60
            )
            if time_to_live <= 0:
                log.warning(
                    "Authentication token has expired. Please generate a new token"
                )
                return False
            else:
                log.info(
                    f"Using valid authentication token with a life of {'{:.2f}'.format(time_to_live)} [HR]"
                )
                return True

    def set_session(self):
        self.session = djsciops_authentication.Session(
            aws_account_id=self.config["aws"]["account_id"],
            s3_role=self.config["s3"]["role"],
            auth_client_id=self.config["djauth"]["client_id"],
        )

    def finish_auth(self):
        widgets.refreshTokenBtn.setEnabled(True)
        log.debug("Worker finished")
        try:
            time = datetime.fromtimestamp(self.session.jwt["exp"]).strftime("%H:%M %p")
            widgets.expTime.setText(f"Token Expiration Time: {time}")
            log.debug("Auth successful")
        except Exception as e:
            log.debug(f"Auth failed - {e}")

    # RESIZE EVENTS
    # ///////////////////////////////////////////////////////////////
    def resizeEvent(self, event):
        # Update Size Grips
        UIFunctions.resize_grips(self)

    # MOUSE CLICK EVENTS
    # ///////////////////////////////////////////////////////////////
    def mousePressEvent(self, event):
        # SET DRAG POS WINDOW
        self.dragPos = event.globalPos()

        # PRINT MOUSE EVENTS
        if event.buttons() == Qt.LeftButton:
            print("Mouse click: LEFT CLICK")
        if event.buttons() == Qt.RightButton:
            print("Mouse click: RIGHT CLICK")


if __name__ == "__main__":
    app = QApplication(sys.argv)
    app.setWindowIcon(QIcon(os.path.join(basedir, "icon.ico")))
    window = MainWindow()
    sys.exit(app.exec())



import appdirs
import pathlib
import os
import yaml
from . import __version__ as version
from . import utils as djsciops_utils

CONFIG_TEMPLATE = """
version: "{djsciops_version}"
aws:
  account_id: "{djsciops_aws_account_id}"
s3:
  role: "{djsciops_s3_role}"
  bucket: "{djsciops_s3_bucket}"
djauth:
  client_id: "{djsciops_djauth_client_id}"
"""

LOG_LEVEL = os.getenv("DJSCIOPS_LOG_LEVEL", "info")


def get_config(stdin_enabled=True):
    from .log import log

    config_directory = appdirs.user_data_dir(appauthor="datajoint", appname="djsciops")
    try:
        # loading existing config
        config = yaml.safe_load(
            pathlib.Path(config_directory, "config.yaml").read_text()
        )
        log.info(
            "Existing configuration detected. Loading from "
            f"{pathlib.Path(config_directory, 'config.yaml')}..."
        )
        return config
    except FileNotFoundError as e:
        if not stdin_enabled:
            raise e
        log.info(
            "Welcome! We've detected that this is your first time using DataJoint "
            "SciOps CLI tools. We'll need to ask a few questions to initialize properly."
        )
        # generate default config
        config = CONFIG_TEMPLATE.format(
            djsciops_aws_account_id=(
                os.getenv("DJSCIOPS_AWS_ACCOUNT_ID")
                if os.getenv("DJSCIOPS_AWS_ACCOUNT_ID")
                else input("\n   -> AWS Account ID? ")
            ),
            djsciops_s3_role=(
                os.getenv("DJSCIOPS_S3_ROLE")
                if os.getenv("DJSCIOPS_S3_ROLE")
                else input("\n   -> S3 Role? ")
            ),
            djsciops_s3_bucket=(
                os.getenv("DJSCIOPS_S3_BUCKET")
                if os.getenv("DJSCIOPS_S3_BUCKET")
                else input("\n   -> S3 Bucket? ")
            ),
            djsciops_djauth_client_id=(
                os.getenv("DJSCIOPS_DJAUTH_CLIENT_ID")
                if os.getenv("DJSCIOPS_DJAUTH_CLIENT_ID")
                else input("\n   -> DataJoint Account Client ID? ")
            ),
            djsciops_version=version,
            djsciops_log_level=os.getenv("DJSCIOPS_LOG_LEVEL", "info"),
        )
        # write config
        os.makedirs(config_directory, exist_ok=True)
        with open(pathlib.Path(config_directory, "config.yaml"), "w") as f:
            f.write(config)

        log.info(
            "Thank you! We've saved your responses to "
            f"{pathlib.Path(config_directory, 'config.yaml')} so you won't need to "
            "specify this again."
        )
        # return config
        return yaml.safe_load(config)



import logging
import io
from . import settings as djsciops_settings

log = logging.getLogger("Primary")
log_format = logging.Formatter(
    "[%(asctime)s][%(funcName)s][%(levelname)s]: %(message)s"
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(log_format)

log.setLevel(level=djsciops_settings.LOG_LEVEL.upper())
log.handlers = [stream_handler]

# https://github.com/tqdm/tqdm/issues/313#issuecomment-267959111
class TqdmToLogger(io.StringIO):
    """
    Output stream for TQDM which will output to logger module instead of
    the StdOut.
    """

    logger = None
    level = None
    buf = ""

    def __init__(self, logger, level=None):
        super(TqdmToLogger, self).__init__()
        self.logger = logger
        self.level = level or logging.INFO

    def write(self, buf):
        self.buf = buf.strip("\r\n\t ")

    def flush(self):
        self.logger.log(self.level, self.buf)



import base64
from datetime import datetime
import json
import boto3
import logging
import flask
import webbrowser
import multiprocessing
import time
import urllib
import http.client
from . import utils as djsciops_utils
from .log import log


def _oidc_login(
    auth_client_id: str,
    auth_url: str = "https://accounts.datajoint.io/auth/auth",
    lookup_service_allowed_origin: str = "https://ops.datajoint.io",
    lookup_service_domain: str = "ops.datajoint.io",
    lookup_service_route: str = "/social-login/api/user",
    lookup_service_auth_provider: str = "accounts.datajoint.io",
    code_challenge: str = "ubNp9Y0Y_FOENQ_Pz3zppyv2yyt0XtJsaPqUgGW9heA",
    code_challenge_method: str = "S256",
    code_verifier: str = "kFn5ZwL6ggOwU1OzKx0E1oZibIMC1ZbMC1WEUXcCV5mFoi015I9nB9CrgUJRkc3oiQT8uBbrvRvVzahM8OS0xJ51XdYaTdAlFeHsb6OZuBPmLD400ozVPrwCE192rtqI",
    callback_port: int = 28282,
    delay_seconds: int = 60,
):
    """
    Primary OIDC login flow.
    """
    # Prepare user
    log.warning(
        "User authentication required to use DataJoint SciOps CLI tools. We'll be "
        "launching a web browser to authenticate your DataJoint account."
    )
    # allocate variables for access and context
    code = None
    cancelled = True
    # Prepare HTTP server to communicate with browser
    logging.getLogger("werkzeug").setLevel(logging.ERROR)
    app = flask.Flask("browser-interface")

    def shutdown_server():
        """
        Shuts down Flask HTTP server.
        """
        func = flask.request.environ.get("werkzeug.server.shutdown")
        if func is not None:
            # Ensure running with the Werkzeug Server
            func()

    @app.route("/login-cancelled")
    def login_cancelled():
        """
        Accepts requests which will cancel the user login.
        """
        shutdown_server()
        return """
        <!doctype html>
        <html>
          <head>
            <script>
              window.onload = function load() {
              window.open('', '_self', '');
              window.close();
              };
            </script>
          </head>
          <body>
          </body>
        </html>
        """

    @app.route("/login-completed")
    def login_completed():
        """
        Redirect after user has successfully logged in.
        """
        nonlocal code
        nonlocal cancelled
        cancelled = False
        code = flask.request.args.get("code")
        shutdown_server()
        return """
        <!doctype html>
        <html>
          <head>
            <script>
              window.onload = function load() {
              window.open('', '_self', '');
              window.close();
              };
            </script>
          </head>
          <body>DataJoint login completed! Feel free to close this tab if it did not close automatically.</body>
        </html>
        """

    # build url
    query_params = dict(
        scope="openid",
        response_type="code",
        client_id=auth_client_id,
        code_challenge=code_challenge,
        code_challenge_method=code_challenge_method,
        redirect_uri=f"http://localhost:{callback_port}/login-completed",
    )
    link = f"{auth_url}?{urllib.parse.urlencode(query_params)}"
    # attempt to launch browser or provide instructions
    browser_available = True
    try:
        webbrowser.get()
    except webbrowser.Error:
        browser_available = False
    if browser_available:
        log.info("Browser available. Launching...")
        webbrowser.open(link, new=2)
    else:
        log.warning(
            "Brower unavailable. On a browser client, please navigate to the "
            f"following link to login: {link}"
        )
    # start response server
    cancel_process = multiprocessing.Process(
        target=_delayed_request,
        kwargs=dict(
            url=f"http://localhost:{callback_port}/login-cancelled",
            delay=delay_seconds,
        ),
    )
    cancel_process.start()
    app.run(host="0.0.0.0", port=callback_port, debug=False)
    cancel_process.terminate()
    # received a response
    if cancelled:
        raise Exception(
            "User login cancelled. User must be logged in to use DataJoint SciOps CLI tools."
        )
    else:
        log.info("User successfully authenticated.")
        # generate user info
        connection = http.client.HTTPSConnection(lookup_service_domain)
        headers = {
            "Content-type": "application/json",
            "Origin": lookup_service_allowed_origin,
        }
        body = json.dumps(
            {
                "auth_provider": lookup_service_auth_provider,
                "redirect_uri": f"http://localhost:{callback_port}/login-completed",
                "code_verifier": code_verifier,
                "client_id": auth_client_id,
                "code": code,
            }
        )
        connection.request("POST", lookup_service_route, body, headers)
        userdata = json.loads(connection.getresponse().read().decode())
        return userdata["access_token"], userdata["username"]


def _delayed_request(*, url: str, delay: str = 0):
    time.sleep(delay)
    return urllib.request.urlopen(url)


def _decode_bearer_token(bearer_token):
    jwt_data = json.loads(
        base64.b64decode(bearer_token.split(".")[1].encode()).decode()
    )
    return jwt_data


class Session:
    def __init__(
        self,
        aws_account_id: str,
        s3_role: str,
        auth_client_id: str,
        bearer_token: str = None,
    ):
        # OAuth2.0 authorization
        if not bearer_token:
            self.bearer_token, self.user = _oidc_login(
                auth_client_id=auth_client_id,
            )
            self.jwt = _decode_bearer_token(self.bearer_token)
        else:
            self.jwt = _decode_bearer_token(self.bearer_token)
            time_to_live = (self.jwt["exp"] - datetime.utcnow().timestamp()) / 60 / 60
            log.info(
                f"Reusing provided bearer token with a life of {time_to_live} [HR]"
            )
            self.bearer_token, self.user = (bearer_token, self.jwt["sub"])
        # AWS temporary credentials
        sts_client = boto3.client(service_name="sts")
        sts_response = sts_client.assume_role_with_web_identity(
            RoleArn=f"arn:aws:iam::{aws_account_id}:role/{s3_role}",
            RoleSessionName=self.user,
            WebIdentityToken=self.bearer_token,
            DurationSeconds=12 * 60 * 60,
        )
        self.aws_access_key_id = sts_response["Credentials"]["AccessKeyId"]
        self.aws_secret_access_key = sts_response["Credentials"]["SecretAccessKey"]
        self.aws_session_token = sts_response["Credentials"]["SessionToken"]
        # AWS resource
        self.s3 = boto3.Session(
            aws_access_key_id=self.aws_access_key_id,
            aws_secret_access_key=self.aws_secret_access_key,
            aws_session_token=self.aws_session_token,
        ).resource("s3")



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

from . custom_grips import CustomGrip



import argparse
import sys
from djsciops import __version__ as version
from djsciops import authentication as djsciops_authentication
from djsciops import axon as djsciops_axon
from djsciops import settings as djsciops_settings


def djsciops(args: list = None):
    """
    Primary console interface for djsciops's shell utilities.

    :param args: List of arguments to be passed in, defaults to reading stdin
    :type args: list, optional
    """
    parser = argparse.ArgumentParser(
        prog="djsciops", description="DataJoint SciOps console interface."
    )
    parser.add_argument(
        "-V", "--version", action="version", version=f"djsciops {version}"
    )
    command = parser.add_subparsers(dest="command")
    axon = command.add_parser("axon", description="Manage object store data.")
    subcommand = axon.add_subparsers(dest="subcommand")
    axon_upload = subcommand.add_parser(
        "upload", description="Copy objects by uploading to object store."
    )
    required_named = axon_upload.add_argument_group("required named arguments")

    required_named.add_argument(
        "source",
        type=str,
        help="Source file or directory on client.",
    )
    required_named.add_argument(
        "destination",
        type=str,
        help="Target directory in object store.",
    )
    kwargs = vars(parser.parse_args(args if sys.argv[1:] else ["-h"]))
    config = djsciops_settings.get_config()
    if kwargs["command"] == "axon" and kwargs["subcommand"] == "upload":
        djsciops_axon.upload_files(
            session=djsciops_authentication.Session(
                aws_account_id=config["aws"]["account_id"],
                s3_role=config["s3"]["role"],
                auth_client_id=config["djauth"]["client_id"],
            ),
            s3_bucket=config["s3"]["bucket"],
            source=kwargs["source"],
            destination=kwargs["destination"],
        )
    raise SystemExit


if __name__ == "__main__":
    djsciops()



class Settings():
    # APP SETTINGS
    # ///////////////////////////////////////////////////////////////
    ENABLE_CUSTOM_TITLE_BAR = True
    MENU_WIDTH = 240
    LEFT_BOX_WIDTH = 275
    CONSOLE_BOX_HEIGHT = 220
    TIME_ANIMATION = 500

    # BTNS LEFT AND RIGHT BOX COLORS
    BTN_LEFT_BOX_COLOR = "background-color: rgb(44, 49, 58);"
    BTN_RIGHT_BOX_COLOR = "background-color: #ff79c6;"

    # MENU SELECTED STYLESHEET
    MENU_SELECTED_STYLESHEET = """
    border-left: 22px solid qlineargradient(spread:pad, x1:0.034, y1:0, x2:0.216, y2:0, stop:0.499 rgba(255, 121, 198, 255), stop:0.5 rgba(85, 170, 255, 0));
    background-color: rgb(40, 44, 52);
    """



import setuptools
import pathlib
import djsciops as package
import re

here = pathlib.Path(__file__).parent.resolve()
with open(pathlib.Path(here, "pip_requirements.txt")) as f:
    requirements = [
        "{pkg} @ {target}#egg={pkg}".format(
            pkg=re.search(r"/([A-Za-z0-9\-]+)\.git", r).group(1), target=r
        )
        if "+" in r
        else r
        for r in f.read().splitlines()
        if "#" not in r
    ]

setuptools.setup(
    name=package.__name__,
    version=package.__version__,
    author="DataJoint",
    author_email="djbot@datajoint.io",
    description="A suite of customer CLI tools for DataJoint SciOps.",
    long_description=pathlib.Path("README.md").read_text(),
    long_description_content_type="text/markdown",
    url="https://github.com/dj-sciops/djsciops-python",
    packages=setuptools.find_packages(exclude=["tests*"]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        "console_scripts": [
            f"{package.__name__}={package.__name__}.command_line:{package.__name__}"
        ],
    },
    package_data={"djsciops.axon_gui": ["*", "*/*", "*/*/*"]},
    include_package_data=True,
    install_requires=requirements,
)



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# MAIN FILE
# ///////////////////////////////////////////////////////////////
from djsciops.axon_gui.main import *

# WITH ACCESS TO MAIN WINDOW WIDGETS
# ///////////////////////////////////////////////////////////////
class AppFunctions(MainWindow):
    def setThemeHack(self):
        Settings.BTN_LEFT_BOX_COLOR = "background-color: #495474;"
        Settings.BTN_RIGHT_BOX_COLOR = "background-color: #495474;"
        Settings.MENU_SELECTED_STYLESHEET = MENU_SELECTED_STYLESHEET = """
        border-left: 22px solid qlineargradient(spread:pad, x1:0.034, y1:0, x2:0.216, y2:0, stop:0.499 rgba(255, 121, 198, 255), stop:0.5 rgba(85, 170, 255, 0));
        background-color: #566388;
        """

        # SET MANUAL STYLES
        # self.ui.lineEdit.setStyleSheet("background-color: #6272a4;")
        # self.ui.pushButton.setStyleSheet("background-color: #6272a4;")
        # self.ui.plainTextEdit.setStyleSheet("background-color: #6272a4;")
        # self.ui.tableWidget.setStyleSheet("QScrollBar:vertical { background: #6272a4; } QScrollBar:horizontal { background: #6272a4; }")
        # self.ui.scrollArea.setStyleSheet("QScrollBar:vertical { background: #6272a4; } QScrollBar:horizontal { background: #6272a4; }")
        # self.ui.comboBox.setStyleSheet("background-color: #6272a4;")
        # self.ui.horizontalScrollBar.setStyleSheet("background-color: #6272a4;")
        # self.ui.verticalScrollBar.setStyleSheet("background-color: #6272a4;")
        # self.ui.commandLinkButton.setStyleSheet("color: #ff79c6;")



import pathlib
import os
import boto3.s3.transfer
import tqdm
from typing import Union
from . import utils as djsciops_utils
import collections.abc
import logging
from .log import log, TqdmToLogger


def _generate_transfer_mapping(
    session,
    s3_bucket: str,
    source: Union[str, list],
    destination: Union[str, list],
) -> tuple:
    if (
        isinstance(source, list)
        and isinstance(destination, list)
        and len(source) == len(destination)
    ):
        log.info("Upload mapping provided.")
        # expect file_paths to be list of Path, object_paths to be list of PurePosixPath
        file_paths, object_paths = (source, destination)
        existing_objects = {
            pathlib.PurePosixPath(op.key)
            for op in session.s3.Bucket(s3_bucket).objects.filter(
                Prefix=os.path.commonprefix(object_paths)
            )
        }
        file_paths, object_paths = tuple(
            zip(
                *(
                    (fp, op)
                    for fp, op in zip(file_paths, object_paths)
                    if op not in existing_objects
                )
            )
        )
        assert file_paths, "All files already exist in object store."
    else:
        # modes: file -> directory, directory -> directory
        assert destination[-1] == "/", "Must point to a directory in object store."
        source = pathlib.Path(source).resolve()
        # recursively list files that are not hidden or directories
        file_paths = {
            fp.relative_to(source)
            for fp in source.rglob("*")
            if not fp.is_dir() and not str(fp.name).startswith(".")
        }
        file_paths = (
            {pathlib.PurePosixPath(_) for _ in file_paths}
            if file_paths
            else {pathlib.PurePosixPath(source.name)}
        )  # if specified a single file
        # recursively list objects
        existing_objects = {
            pathlib.PurePosixPath(op.key.replace(destination, ""))
            for op in session.s3.Bucket(s3_bucket).objects.filter(Prefix=destination)
        }
        # exclude objects that exist and verify that new objects are present
        file_paths = file_paths - existing_objects
        assert file_paths, "All files already exist in object store."
        object_paths = [pathlib.PurePosixPath(destination, fp) for fp in file_paths]
        file_paths = [
            pathlib.Path(source if source.is_dir() else source.parent, fp)
            for fp in file_paths
        ]

    return file_paths, object_paths


def list_files(
    session,
    s3_bucket: str,
    s3_prefix: str,
):
    objects = [
        dict(key=op.key, _size=op.size)
        for op in session.s3.Bucket(s3_bucket).objects.filter(Prefix=s3_prefix)
    ]
    objects = sorted(objects, key=lambda o: o["key"])

    tree = {}
    for o in objects:
        node = tree  # local node
        levels = o["key"].split("/")
        for level in levels:
            node = node.setdefault(level, dict())
            node["_size"] = (
                o["_size"] + node["_size"] if "_size" in node else o["_size"]
            )

    def update(d):
        for k, v in d.items():
            if isinstance(v, collections.abc.Mapping) and len(v) > 1:
                if k != "_size":
                    size = d[k].pop("_size")
                    d[k] = update(d.get(k, {}))
            else:
                d[k] = None
        return d

    return update(tree)


def upload_files(
    session,
    s3_bucket: str,
    source: Union[str, list],
    destination: Union[str, list],
):
    file_paths, object_paths = _generate_transfer_mapping(
        session=session,
        s3_bucket=s3_bucket,
        source=source,
        destination=destination,
    )
    log.info("Starting upload")
    with tqdm.tqdm(
        total=sum(os.stat(fp).st_size for fp in file_paths),
        unit="B",
        unit_scale=True,
        desc="",
        file=TqdmToLogger(log, level=logging.INFO),
    ) as pbar:
        for fp, op in zip(file_paths, object_paths):
            log.info(f"{fp}->{op}")
            ## hash metadata
            contents_hash = djsciops_utils.uuid_from_file(fp)
            log.debug(f"contents_hash: {contents_hash.hex}")
            ## upload
            session.s3.Bucket(s3_bucket).upload_file(
                Filename=str(fp),
                Key=str(op),
                Config=boto3.s3.transfer.TransferConfig(
                    multipart_threshold=1024 * 25,
                    max_concurrency=10,
                    multipart_chunksize=1024 * 25,
                    use_threads=True,
                ),
                Callback=lambda bytes_transferred: pbar.update(bytes_transferred),
                ExtraArgs={"Metadata": {"contents_hash": contents_hash.hex}},
            )



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QGridLayout, QHBoxLayout,
    QHeaderView, QLabel, QLayout, QLineEdit,
    QMainWindow, QPushButton, QSizePolicy, QStackedWidget,
    QTextEdit, QTreeWidget, QTreeWidgetItem, QVBoxLayout,
    QWidget)
from . resources_rc import *

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1280, 720)
        MainWindow.setMinimumSize(QSize(940, 560))
        self.styleSheet = QWidget(MainWindow)
        self.styleSheet.setObjectName(u"styleSheet")
        font = QFont()
        font.setFamilies([u"Segoe UI"])
        font.setPointSize(10)
        font.setBold(False)
        font.setItalic(False)
        self.styleSheet.setFont(font)
        self.styleSheet.setStyleSheet(u"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"\n"
"SET APP STYLESHEET - FULL STYLES HERE\n"
"DARK THEME - DRACULA COLOR BASED\n"
"\n"
"///////////////////////////////////////////////////////////////////////////////////////////////// */\n"
"\n"
"QWidget{\n"
"	color: rgb(221, 221, 221);\n"
"	font: 10pt \"Segoe UI\";\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Tooltip */\n"
"QToolTip {\n"
"	color: #ffffff;\n"
"	background-color: rgba(33, 37, 43, 180);\n"
"	border: 1px solid rgb(44, 49, 58);\n"
"	background-image: none;\n"
"	background-position: left center;\n"
"    background-repeat: no-repeat;\n"
"	border: none;\n"
"	border-left: 2px solid rgb(255, 121, 198);\n"
"	text-align: left;\n"
"	padding-left: 8px;\n"
"	margin: 0px;\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Bg App */\n"
"#bgApp {	\n"
"	background"
                        "-color: rgb(40, 44, 52);\n"
"	border: 1px solid rgb(44, 49, 58);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Left Menu */\n"
"#leftMenuBg {	\n"
"	background-color: rgb(33, 37, 43);\n"
"}\n"
"#topLogo {\n"
"	background-color: rgb(33, 37, 43);\n"
"	background-image: url(:/images/images/images/symbol.png);\n"
"	background-position: centered;\n"
"	background-repeat: no-repeat;\n"
"}\n"
"#titleLeftApp { font: 63 12pt \"Segoe UI Semibold\"; }\n"
"#titleLeftDescription { font: 8pt \"Segoe UI\"; color: rgb(189, 147, 249); }\n"
"\n"
"/* MENUS */\n"
"#topMenu .QPushButton {	\n"
"	background-position: left center;\n"
"    background-repeat: no-repeat;\n"
"	border: none;\n"
"	border-left: 22px solid transparent;\n"
"	background-color: transparent;\n"
"	text-align: left;\n"
"	padding-left: 44px;\n"
"}\n"
"#topMenu .QPushButton:hover {\n"
"	background-color: rgb(40, 44, 52);\n"
"}\n"
"#topMenu .QPushButton:pressed {	\n"
"	background-color: rgb(189, "
                        "147, 249);\n"
"	color: rgb(255, 255, 255);\n"
"}\n"
"#bottomMenu .QPushButton {	\n"
"	background-position: left center;\n"
"    background-repeat: no-repeat;\n"
"	border: none;\n"
"	border-left: 20px solid transparent;\n"
"	background-color:transparent;\n"
"	text-align: left;\n"
"	padding-left: 44px;\n"
"}\n"
"#bottomMenu .QPushButton:hover {\n"
"	background-color: rgb(40, 44, 52);\n"
"}\n"
"#bottomMenu .QPushButton:pressed {	\n"
"	background-color: rgb(189, 147, 249);\n"
"	color: rgb(255, 255, 255);\n"
"}\n"
"#leftMenuFrame{\n"
"	border-top: 3px solid rgb(44, 49, 58);\n"
"}\n"
"\n"
"/* Toggle Button */\n"
"#toggleButton {\n"
"	background-position: left center;\n"
"    background-repeat: no-repeat;\n"
"	border: none;\n"
"	border-left: 20px solid transparent;\n"
"	background-color: rgb(37, 41, 48);\n"
"	text-align: left;\n"
"	padding-left: 44px;\n"
"	color: rgb(113, 126, 149);\n"
"}\n"
"#toggleButton:hover {\n"
"	background-color: rgb(40, 44, 52);\n"
"}\n"
"#toggleButton:pressed {\n"
"	background-color: rgb(189"
                        ", 147, 249);\n"
"}\n"
"\n"
"/* Title Menu */\n"
"#titleRightInfo { padding-left: 10px; }\n"
"\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Extra Tab */\n"
"#extraLeftBox {	\n"
"	background-color: rgb(44, 49, 58);\n"
"}\n"
"#extraTopBg{	\n"
"	background-color: rgb(189, 147, 249)\n"
"}\n"
"\n"
"/* Icon */\n"
"#extraIcon {\n"
"	background-position: center;\n"
"	background-repeat: no-repeat;\n"
"	background-image: url(:/icons/images/icons/icon_settings.png);\n"
"}\n"
"\n"
"/* Label */\n"
"#extraLabel { color: rgb(255, 255, 255); }\n"
"\n"
"/* Btn Close */\n"
"#extraCloseColumnBtn { background-color: rgba(255, 255, 255, 0); border: none;  border-radius: 5px; }\n"
"#extraCloseColumnBtn:hover { background-color: rgb(196, 161, 249); border-style: solid; border-radius: 4px; }\n"
"#extraCloseColumnBtn:pressed { background-color: rgb(180, 141, 238); border-style: solid; border-radius: 4px; }\n"
"\n"
"/* Extra Content */\n"
"#extraContent{\n"
"	border-to"
                        "p: 3px solid rgb(40, 44, 52);\n"
"}\n"
"\n"
"/* Extra Top Menus */\n"
"#extraTopMenu .QPushButton {\n"
"background-position: left center;\n"
"    background-repeat: no-repeat;\n"
"	border: none;\n"
"	border-left: 22px solid transparent;\n"
"	background-color:transparent;\n"
"	text-align: left;\n"
"	padding-left: 44px;\n"
"}\n"
"#extraTopMenu .QPushButton:hover {\n"
"	background-color: rgb(40, 44, 52);\n"
"}\n"
"#extraTopMenu .QPushButton:pressed {	\n"
"	background-color: rgb(189, 147, 249);\n"
"	color: rgb(255, 255, 255);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Content App */\n"
"#contentTopBg{	\n"
"	background-color: rgb(33, 37, 43);\n"
"}\n"
"#contentBottom{\n"
"	border-top: 3px solid rgb(44, 49, 58);\n"
"}\n"
"\n"
"/* Top Buttons */\n"
"#rightButtons .QPushButton { background-color: rgba(255, 255, 255, 0); border: none;  border-radius: 5px; }\n"
"#rightButtons .QPushButton:hover { background-color: rgb(44, 49, 57); border-style:"
                        " solid; border-radius: 4px; }\n"
"#rightButtons .QPushButton:pressed { background-color: rgb(23, 26, 30); border-style: solid; border-radius: 4px; }\n"
"\n"
"/* Theme Settings */\n"
"#extraRightBox { background-color: rgb(44, 49, 58); }\n"
"#themeSettingsTopDetail { background-color: rgb(189, 147, 249); }\n"
"\n"
"/* Bottom Bar */\n"
"#bottomBar { background-color: rgb(44, 49, 58); }\n"
"#bottomBar QLabel { font-size: 11px; color: rgb(113, 126, 149); padding-left: 10px; padding-right: 10px; padding-bottom: 2px; }\n"
"\n"
"/* CONTENT SETTINGS */\n"
"/* MENUS */\n"
"#contentSettings .QPushButton {	\n"
"	background-position: left center;\n"
"    background-repeat: no-repeat;\n"
"	border: none;\n"
"	border-left: 22px solid transparent;\n"
"	background-color:transparent;\n"
"	text-align: left;\n"
"	padding-left: 44px;\n"
"}\n"
"#contentSettings .QPushButton:hover {\n"
"	background-color: rgb(40, 44, 52);\n"
"}\n"
"#contentSettings .QPushButton:pressed {	\n"
"	background-color: rgb(189, 147, 249);\n"
"	color: rgb(25"
                        "5, 255, 255);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"QTableWidget */\n"
"QTableWidget {	\n"
"	background-color: transparent;\n"
"	padding: 10px;\n"
"	border-radius: 5px;\n"
"	gridline-color: rgb(44, 49, 58);\n"
"	border-bottom: 1px solid rgb(44, 49, 60);\n"
"}\n"
"QTableWidget::item{\n"
"	border-color: rgb(44, 49, 60);\n"
"	padding-left: 5px;\n"
"	padding-right: 5px;\n"
"	gridline-color: rgb(44, 49, 60);\n"
"}\n"
"QTableWidget::item:selected{\n"
"	background-color: rgb(189, 147, 249);\n"
"}\n"
"QHeaderView::section{\n"
"	background-color: rgb(33, 37, 43);\n"
"	max-width: 30px;\n"
"	border: 1px solid rgb(44, 49, 58);\n"
"	border-style: none;\n"
"    border-bottom: 1px solid rgb(44, 49, 60);\n"
"    border-right: 1px solid rgb(44, 49, 60);\n"
"}\n"
"QTableWidget::horizontalHeader {	\n"
"	background-color: rgb(33, 37, 43);\n"
"}\n"
"QHeaderView::section:horizontal\n"
"{\n"
"    border: 1px solid rgb(33, 37, 43);\n"
"	background-color"
                        ": rgb(33, 37, 43);\n"
"	padding: 3px;\n"
"	border-top-left-radius: 7px;\n"
"    border-top-right-radius: 7px;\n"
"}\n"
"QHeaderView::section:vertical\n"
"{\n"
"    border: 1px solid rgb(44, 49, 60);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"LineEdit */\n"
"QLineEdit {\n"
"	background-color: rgb(33, 37, 43);\n"
"	border-radius: 5px;\n"
"	border: 2px solid rgb(33, 37, 43);\n"
"	padding-left: 10px;\n"
"	selection-color: rgb(255, 255, 255);\n"
"	selection-background-color: rgb(255, 121, 198);\n"
"}\n"
"QLineEdit:hover {\n"
"	border: 2px solid rgb(64, 71, 88);\n"
"}\n"
"QLineEdit:focus {\n"
"	border: 2px solid rgb(91, 101, 124);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"PlainTextEdit */\n"
"QPlainTextEdit {\n"
"	background-color: rgb(27, 29, 35);\n"
"	border-radius: 5px;\n"
"	padding: 10px;\n"
"	selection-color: rgb(255, 255, 255);\n"
"	selection-background-colo"
                        "r: rgb(255, 121, 198);\n"
"}\n"
"QPlainTextEdit  QScrollBar:vertical {\n"
"    width: 8px;\n"
" }\n"
"QPlainTextEdit  QScrollBar:horizontal {\n"
"    height: 8px;\n"
" }\n"
"QPlainTextEdit:hover {\n"
"	border: 2px solid rgb(64, 71, 88);\n"
"}\n"
"QPlainTextEdit:focus {\n"
"	border: 2px solid rgb(91, 101, 124);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"ScrollBars */\n"
"QScrollBar:horizontal {\n"
"    border: none;\n"
"    background: rgb(52, 59, 72);\n"
"    height: 8px;\n"
"    margin: 0px 21px 0 21px;\n"
"	border-radius: 0px;\n"
"}\n"
"QScrollBar::handle:horizontal {\n"
"    background: rgb(189, 147, 249);\n"
"    min-width: 25px;\n"
"	border-radius: 4px\n"
"}\n"
"QScrollBar::add-line:horizontal {\n"
"    border: none;\n"
"    background: rgb(55, 63, 77);\n"
"    width: 20px;\n"
"	border-top-right-radius: 4px;\n"
"    border-bottom-right-radius: 4px;\n"
"    subcontrol-position: right;\n"
"    subcontrol-origin: margin;\n"
"}\n"
"Q"
                        "ScrollBar::sub-line:horizontal {\n"
"    border: none;\n"
"    background: rgb(55, 63, 77);\n"
"    width: 20px;\n"
"	border-top-left-radius: 4px;\n"
"    border-bottom-left-radius: 4px;\n"
"    subcontrol-position: left;\n"
"    subcontrol-origin: margin;\n"
"}\n"
"QScrollBar::up-arrow:horizontal, QScrollBar::down-arrow:horizontal\n"
"{\n"
"     background: none;\n"
"}\n"
"QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal\n"
"{\n"
"     background: none;\n"
"}\n"
" QScrollBar:vertical {\n"
"	border: none;\n"
"    background: rgb(52, 59, 72);\n"
"    width: 8px;\n"
"    margin: 21px 0 21px 0;\n"
"	border-radius: 0px;\n"
" }\n"
" QScrollBar::handle:vertical {	\n"
"	background: rgb(189, 147, 249);\n"
"    min-height: 25px;\n"
"	border-radius: 4px\n"
" }\n"
" QScrollBar::add-line:vertical {\n"
"     border: none;\n"
"    background: rgb(55, 63, 77);\n"
"     height: 20px;\n"
"	border-bottom-left-radius: 4px;\n"
"    border-bottom-right-radius: 4px;\n"
"     subcontrol-position: bottom;\n"
"     sub"
                        "control-origin: margin;\n"
" }\n"
" QScrollBar::sub-line:vertical {\n"
"	border: none;\n"
"    background: rgb(55, 63, 77);\n"
"     height: 20px;\n"
"	border-top-left-radius: 4px;\n"
"    border-top-right-radius: 4px;\n"
"     subcontrol-position: top;\n"
"     subcontrol-origin: margin;\n"
" }\n"
" QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {\n"
"     background: none;\n"
" }\n"
"\n"
" QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {\n"
"     background: none;\n"
" }\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"CheckBox */\n"
"QCheckBox::indicator {\n"
"    border: 3px solid rgb(52, 59, 72);\n"
"	width: 15px;\n"
"	height: 15px;\n"
"	border-radius: 10px;\n"
"    background: rgb(44, 49, 60);\n"
"}\n"
"QCheckBox::indicator:hover {\n"
"    border: 3px solid rgb(58, 66, 81);\n"
"}\n"
"QCheckBox::indicator:checked {\n"
"    background: 3px solid rgb(52, 59, 72);\n"
"	border: 3px solid rgb(52, 59, 72);	\n"
"	backg"
                        "round-image: url(:/icons/images/icons/cil-check-alt.png);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"RadioButton */\n"
"QRadioButton::indicator {\n"
"    border: 3px solid rgb(52, 59, 72);\n"
"	width: 15px;\n"
"	height: 15px;\n"
"	border-radius: 10px;\n"
"    background: rgb(44, 49, 60);\n"
"}\n"
"QRadioButton::indicator:hover {\n"
"    border: 3px solid rgb(58, 66, 81);\n"
"}\n"
"QRadioButton::indicator:checked {\n"
"    background: 3px solid rgb(94, 106, 130);\n"
"	border: 3px solid rgb(52, 59, 72);	\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"ComboBox */\n"
"QComboBox{\n"
"	background-color: rgb(27, 29, 35);\n"
"	border-radius: 5px;\n"
"	border: 2px solid rgb(33, 37, 43);\n"
"	padding: 5px;\n"
"	padding-left: 10px;\n"
"}\n"
"QComboBox:hover{\n"
"	border: 2px solid rgb(64, 71, 88);\n"
"}\n"
"QComboBox::drop-down {\n"
"	subcontrol-origin: padding;\n"
"	subcon"
                        "trol-position: top right;\n"
"	width: 25px; \n"
"	border-left-width: 3px;\n"
"	border-left-color: rgba(39, 44, 54, 150);\n"
"	border-left-style: solid;\n"
"	border-top-right-radius: 3px;\n"
"	border-bottom-right-radius: 3px;	\n"
"	background-image: url(:/icons/images/icons/cil-arrow-bottom.png);\n"
"	background-position: center;\n"
"	background-repeat: no-reperat;\n"
" }\n"
"QComboBox QAbstractItemView {\n"
"	color: rgb(255, 121, 198);	\n"
"	background-color: rgb(33, 37, 43);\n"
"	padding: 10px;\n"
"	selection-background-color: rgb(39, 44, 54);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Sliders */\n"
"QSlider::groove:horizontal {\n"
"    border-radius: 5px;\n"
"    height: 10px;\n"
"	margin: 0px;\n"
"	background-color: rgb(52, 59, 72);\n"
"}\n"
"QSlider::groove:horizontal:hover {\n"
"	background-color: rgb(55, 62, 76);\n"
"}\n"
"QSlider::handle:horizontal {\n"
"    background-color: rgb(189, 147, 249);\n"
"    border: none;\n"
"    he"
                        "ight: 10px;\n"
"    width: 10px;\n"
"    margin: 0px;\n"
"	border-radius: 5px;\n"
"}\n"
"QSlider::handle:horizontal:hover {\n"
"    background-color: rgb(195, 155, 255);\n"
"}\n"
"QSlider::handle:horizontal:pressed {\n"
"    background-color: rgb(255, 121, 198);\n"
"}\n"
"\n"
"QSlider::groove:vertical {\n"
"    border-radius: 5px;\n"
"    width: 10px;\n"
"    margin: 0px;\n"
"	background-color: rgb(52, 59, 72);\n"
"}\n"
"QSlider::groove:vertical:hover {\n"
"	background-color: rgb(55, 62, 76);\n"
"}\n"
"QSlider::handle:vertical {\n"
"    background-color: rgb(189, 147, 249);\n"
"	border: none;\n"
"    height: 10px;\n"
"    width: 10px;\n"
"    margin: 0px;\n"
"	border-radius: 5px;\n"
"}\n"
"QSlider::handle:vertical:hover {\n"
"    background-color: rgb(195, 155, 255);\n"
"}\n"
"QSlider::handle:vertical:pressed {\n"
"    background-color: rgb(255, 121, 198);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"CommandLinkButton */\n"
"QCommandLin"
                        "kButton {	\n"
"	color: rgb(255, 121, 198);\n"
"	border-radius: 5px;\n"
"	padding: 5px;\n"
"	color: rgb(255, 170, 255);\n"
"}\n"
"QCommandLinkButton:hover {	\n"
"	color: rgb(255, 170, 255);\n"
"	background-color: rgb(44, 49, 60);\n"
"}\n"
"QCommandLinkButton:pressed {	\n"
"	color: rgb(189, 147, 249);\n"
"	background-color: rgb(52, 58, 71);\n"
"}\n"
"\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"Button */\n"
"#pagesContainer QPushButton {\n"
"	border: 2px solid rgb(52, 59, 72);\n"
"	border-radius: 5px;	\n"
"	background-color: rgb(52, 59, 72);\n"
"}\n"
"#pagesContainer QPushButton:hover {\n"
"	background-color: rgb(57, 65, 80);\n"
"	border: 2px solid rgb(61, 70, 86);\n"
"}\n"
"#pagesContainer QPushButton:pressed {	\n"
"	background-color: rgb(35, 40, 49);\n"
"	border: 2px solid rgb(43, 50, 61);\n"
"}\n"
"\n"
"")
        self.appMargins = QVBoxLayout(self.styleSheet)
        self.appMargins.setSpacing(0)
        self.appMargins.setObjectName(u"appMargins")
        self.appMargins.setContentsMargins(10, 10, 10, 10)
        self.bgApp = QFrame(self.styleSheet)
        self.bgApp.setObjectName(u"bgApp")
        self.bgApp.setStyleSheet(u"")
        self.bgApp.setFrameShape(QFrame.NoFrame)
        self.bgApp.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_6 = QHBoxLayout(self.bgApp)
        self.horizontalLayout_6.setSpacing(0)
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.horizontalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.leftMenuBg = QFrame(self.bgApp)
        self.leftMenuBg.setObjectName(u"leftMenuBg")
        self.leftMenuBg.setMinimumSize(QSize(60, 0))
        self.leftMenuBg.setMaximumSize(QSize(60, 16777215))
        self.leftMenuBg.setFrameShape(QFrame.NoFrame)
        self.leftMenuBg.setFrameShadow(QFrame.Raised)
        self.verticalLayout_3 = QVBoxLayout(self.leftMenuBg)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.topLogoInfo = QFrame(self.leftMenuBg)
        self.topLogoInfo.setObjectName(u"topLogoInfo")
        self.topLogoInfo.setMinimumSize(QSize(0, 50))
        self.topLogoInfo.setMaximumSize(QSize(16777215, 50))
        self.topLogoInfo.setFrameShape(QFrame.NoFrame)
        self.topLogoInfo.setFrameShadow(QFrame.Raised)
        self.topLogo = QFrame(self.topLogoInfo)
        self.topLogo.setObjectName(u"topLogo")
        self.topLogo.setGeometry(QRect(10, 5, 42, 42))
        self.topLogo.setMinimumSize(QSize(42, 42))
        self.topLogo.setMaximumSize(QSize(42, 42))
        self.topLogo.setFrameShape(QFrame.NoFrame)
        self.topLogo.setFrameShadow(QFrame.Raised)
        self.titleLeftApp = QLabel(self.topLogoInfo)
        self.titleLeftApp.setObjectName(u"titleLeftApp")
        self.titleLeftApp.setGeometry(QRect(70, 8, 160, 20))
        font1 = QFont()
        font1.setFamilies([u"Segoe UI Semibold"])
        font1.setPointSize(12)
        font1.setBold(False)
        font1.setItalic(False)
        self.titleLeftApp.setFont(font1)
        self.titleLeftApp.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)
        self.titleLeftDescription = QLabel(self.topLogoInfo)
        self.titleLeftDescription.setObjectName(u"titleLeftDescription")
        self.titleLeftDescription.setGeometry(QRect(70, 27, 160, 16))
        self.titleLeftDescription.setMaximumSize(QSize(16777215, 16))
        font2 = QFont()
        font2.setFamilies([u"Segoe UI"])
        font2.setPointSize(8)
        font2.setBold(False)
        font2.setItalic(False)
        self.titleLeftDescription.setFont(font2)
        self.titleLeftDescription.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)

        self.verticalLayout_3.addWidget(self.topLogoInfo)

        self.leftMenuFrame = QFrame(self.leftMenuBg)
        self.leftMenuFrame.setObjectName(u"leftMenuFrame")
        self.leftMenuFrame.setFrameShape(QFrame.NoFrame)
        self.leftMenuFrame.setFrameShadow(QFrame.Raised)
        self.verticalLayout_10 = QVBoxLayout(self.leftMenuFrame)
        self.verticalLayout_10.setSpacing(0)
        self.verticalLayout_10.setObjectName(u"verticalLayout_10")
        self.verticalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.toggleBox = QFrame(self.leftMenuFrame)
        self.toggleBox.setObjectName(u"toggleBox")
        self.toggleBox.setMaximumSize(QSize(16777215, 45))
        self.toggleBox.setFrameShape(QFrame.NoFrame)
        self.toggleBox.setFrameShadow(QFrame.Raised)
        self.verticalLayout_4 = QVBoxLayout(self.toggleBox)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.toggleButton = QPushButton(self.toggleBox)
        self.toggleButton.setObjectName(u"toggleButton")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toggleButton.sizePolicy().hasHeightForWidth())
        self.toggleButton.setSizePolicy(sizePolicy)
        self.toggleButton.setMinimumSize(QSize(0, 45))
        self.toggleButton.setFont(font)
        self.toggleButton.setCursor(QCursor(Qt.PointingHandCursor))
        self.toggleButton.setLayoutDirection(Qt.LeftToRight)
        self.toggleButton.setStyleSheet(u"background-image: url(:/icons/images/icons/icon_menu.png);")

        self.verticalLayout_4.addWidget(self.toggleButton)


        self.verticalLayout_10.addWidget(self.toggleBox)

        self.topMenu = QFrame(self.leftMenuFrame)
        self.topMenu.setObjectName(u"topMenu")
        self.topMenu.setFrameShape(QFrame.NoFrame)
        self.topMenu.setFrameShadow(QFrame.Raised)
        self.verticalLayout_8 = QVBoxLayout(self.topMenu)
        self.verticalLayout_8.setSpacing(0)
        self.verticalLayout_8.setObjectName(u"verticalLayout_8")
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.btn_upload = QPushButton(self.topMenu)
        self.btn_upload.setObjectName(u"btn_upload")
        sizePolicy.setHeightForWidth(self.btn_upload.sizePolicy().hasHeightForWidth())
        self.btn_upload.setSizePolicy(sizePolicy)
        self.btn_upload.setMinimumSize(QSize(0, 45))
        self.btn_upload.setFont(font)
        self.btn_upload.setCursor(QCursor(Qt.PointingHandCursor))
        self.btn_upload.setLayoutDirection(Qt.LeftToRight)
        self.btn_upload.setStyleSheet(u"background-image: url(:/icons/images/icons/cil-cloud-upload.png);")

        self.verticalLayout_8.addWidget(self.btn_upload)

        self.btn_bucket = QPushButton(self.topMenu)
        self.btn_bucket.setObjectName(u"btn_bucket")
        sizePolicy.setHeightForWidth(self.btn_bucket.sizePolicy().hasHeightForWidth())
        self.btn_bucket.setSizePolicy(sizePolicy)
        self.btn_bucket.setMinimumSize(QSize(0, 45))
        self.btn_bucket.setFont(font)
        self.btn_bucket.setCursor(QCursor(Qt.PointingHandCursor))
        self.btn_bucket.setLayoutDirection(Qt.LeftToRight)
        self.btn_bucket.setStyleSheet(u"background-image: url(:/icons/images/icons/cil-paint-bucket.png);")

        self.verticalLayout_8.addWidget(self.btn_bucket)


        self.verticalLayout_10.addWidget(self.topMenu, 0, Qt.AlignTop)

        self.bottomMenu = QFrame(self.leftMenuFrame)
        self.bottomMenu.setObjectName(u"bottomMenu")
        self.bottomMenu.setFrameShape(QFrame.NoFrame)
        self.bottomMenu.setFrameShadow(QFrame.Raised)
        self.verticalLayout_9 = QVBoxLayout(self.bottomMenu)
        self.verticalLayout_9.setSpacing(0)
        self.verticalLayout_9.setObjectName(u"verticalLayout_9")
        self.verticalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.toggleLeftBox = QPushButton(self.bottomMenu)
        self.toggleLeftBox.setObjectName(u"toggleLeftBox")
        sizePolicy.setHeightForWidth(self.toggleLeftBox.sizePolicy().hasHeightForWidth())
        self.toggleLeftBox.setSizePolicy(sizePolicy)
        self.toggleLeftBox.setMinimumSize(QSize(0, 45))
        self.toggleLeftBox.setFont(font)
        self.toggleLeftBox.setCursor(QCursor(Qt.PointingHandCursor))
        self.toggleLeftBox.setLayoutDirection(Qt.LeftToRight)
        self.toggleLeftBox.setStyleSheet(u"background-image: url(:/icons/images/icons/icon_settings.png);")

        self.verticalLayout_9.addWidget(self.toggleLeftBox)


        self.verticalLayout_10.addWidget(self.bottomMenu, 0, Qt.AlignBottom)


        self.verticalLayout_3.addWidget(self.leftMenuFrame)


        self.horizontalLayout_6.addWidget(self.leftMenuBg)

        self.extraLeftBox = QFrame(self.bgApp)
        self.extraLeftBox.setObjectName(u"extraLeftBox")
        sizePolicy1 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.extraLeftBox.sizePolicy().hasHeightForWidth())
        self.extraLeftBox.setSizePolicy(sizePolicy1)
        self.extraLeftBox.setMinimumSize(QSize(0, 0))
        self.extraLeftBox.setMaximumSize(QSize(0, 16777215))
        self.extraLeftBox.setFrameShape(QFrame.NoFrame)
        self.extraLeftBox.setFrameShadow(QFrame.Raised)
        self.extraColumLayout = QVBoxLayout(self.extraLeftBox)
        self.extraColumLayout.setSpacing(0)
        self.extraColumLayout.setObjectName(u"extraColumLayout")
        self.extraColumLayout.setContentsMargins(0, 0, 0, 0)
        self.extraTopBg = QFrame(self.extraLeftBox)
        self.extraTopBg.setObjectName(u"extraTopBg")
        self.extraTopBg.setMinimumSize(QSize(0, 50))
        self.extraTopBg.setMaximumSize(QSize(16777215, 50))
        self.extraTopBg.setFrameShape(QFrame.NoFrame)
        self.extraTopBg.setFrameShadow(QFrame.Raised)
        self.verticalLayout_11 = QVBoxLayout(self.extraTopBg)
        self.verticalLayout_11.setSpacing(0)
        self.verticalLayout_11.setObjectName(u"verticalLayout_11")
        self.verticalLayout_11.setContentsMargins(0, 0, 0, 0)
        self.extraTopLayout = QGridLayout()
        self.extraTopLayout.setObjectName(u"extraTopLayout")
        self.extraTopLayout.setHorizontalSpacing(10)
        self.extraTopLayout.setVerticalSpacing(0)
        self.extraTopLayout.setContentsMargins(10, -1, 10, -1)
        self.extraIcon = QFrame(self.extraTopBg)
        self.extraIcon.setObjectName(u"extraIcon")
        self.extraIcon.setMinimumSize(QSize(20, 0))
        self.extraIcon.setMaximumSize(QSize(20, 20))
        self.extraIcon.setFrameShape(QFrame.NoFrame)
        self.extraIcon.setFrameShadow(QFrame.Raised)

        self.extraTopLayout.addWidget(self.extraIcon, 0, 0, 1, 1)

        self.extraLabel = QLabel(self.extraTopBg)
        self.extraLabel.setObjectName(u"extraLabel")
        self.extraLabel.setMinimumSize(QSize(150, 0))

        self.extraTopLayout.addWidget(self.extraLabel, 0, 1, 1, 1)

        self.extraCloseColumnBtn = QPushButton(self.extraTopBg)
        self.extraCloseColumnBtn.setObjectName(u"extraCloseColumnBtn")
        self.extraCloseColumnBtn.setMinimumSize(QSize(28, 28))
        self.extraCloseColumnBtn.setMaximumSize(QSize(28, 28))
        self.extraCloseColumnBtn.setCursor(QCursor(Qt.PointingHandCursor))
        icon = QIcon()
        icon.addFile(u":/icons/images/icons/icon_close.png", QSize(), QIcon.Normal, QIcon.Off)
        self.extraCloseColumnBtn.setIcon(icon)
        self.extraCloseColumnBtn.setIconSize(QSize(20, 20))

        self.extraTopLayout.addWidget(self.extraCloseColumnBtn, 0, 2, 1, 1)


        self.verticalLayout_11.addLayout(self.extraTopLayout)


        self.extraColumLayout.addWidget(self.extraTopBg)

        self.extraContent = QFrame(self.extraLeftBox)
        self.extraContent.setObjectName(u"extraContent")
        self.extraContent.setFrameShape(QFrame.NoFrame)
        self.extraContent.setFrameShadow(QFrame.Raised)
        self.verticalLayout_12 = QVBoxLayout(self.extraContent)
        self.verticalLayout_12.setSpacing(0)
        self.verticalLayout_12.setObjectName(u"verticalLayout_12")
        self.verticalLayout_12.setContentsMargins(12, 12, 12, 12)
        self.row_1 = QFrame(self.extraContent)
        self.row_1.setObjectName(u"row_1")
        self.row_1.setMinimumSize(QSize(0, 0))
        self.row_1.setLayoutDirection(Qt.LeftToRight)
        self.row_1.setAutoFillBackground(False)
        self.row_1.setStyleSheet(u"border: 0;")
        self.row_1.setFrameShape(QFrame.StyledPanel)
        self.row_1.setFrameShadow(QFrame.Raised)
        self.gridLayout_2 = QGridLayout(self.row_1)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setHorizontalSpacing(0)
        self.gridLayout_2.setVerticalSpacing(10)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.aws_acc_id = QLineEdit(self.row_1)
        self.aws_acc_id.setObjectName(u"aws_acc_id")
        sizePolicy.setHeightForWidth(self.aws_acc_id.sizePolicy().hasHeightForWidth())
        self.aws_acc_id.setSizePolicy(sizePolicy)
        self.aws_acc_id.setMinimumSize(QSize(200, 30))
        self.aws_acc_id.setStyleSheet(u"QLineEdit {\n"
"	background-color: rgb(33, 37, 43);\n"
"	border-radius: 5px;\n"
"	border: 2px solid rgb(33, 37, 43);\n"
"	padding-left: 10px;\n"
"	selection-color: rgb(255, 255, 255);\n"
"	selection-background-color: rgb(255, 121, 198);\n"
"}\n"
"QLineEdit:hover {\n"
"	border: 2px solid rgb(64, 71, 88);\n"
"}\n"
"QLineEdit:focus {\n"
"	border: 2px solid rgb(91, 101, 124);\n"
"}")

        self.gridLayout_2.addWidget(self.aws_acc_id, 1, 0, 1, 1)

        self.labelVersion_4 = QLabel(self.row_1)
        self.labelVersion_4.setObjectName(u"labelVersion_4")
        sizePolicy.setHeightForWidth(self.labelVersion_4.sizePolicy().hasHeightForWidth())
        self.labelVersion_4.setSizePolicy(sizePolicy)
        self.labelVersion_4.setMinimumSize(QSize(0, 0))
        font3 = QFont()
        font3.setFamilies([u"Segoe UI"])
        font3.setBold(False)
        font3.setItalic(False)
        self.labelVersion_4.setFont(font3)
        self.labelVersion_4.setStyleSheet(u"color: #f8f8f2; font: 14px")
        self.labelVersion_4.setLineWidth(1)
        self.labelVersion_4.setAlignment(Qt.AlignCenter)

        self.gridLayout_2.addWidget(self.labelVersion_4, 0, 0, 1, 1)


        self.verticalLayout_12.addWidget(self.row_1, 0, Qt.AlignVCenter)

        self.row_2 = QFrame(self.extraContent)
        self.row_2.setObjectName(u"row_2")
        self.row_2.setStyleSheet(u"border: 0;")
        self.row_2.setFrameShape(QFrame.StyledPanel)
        self.row_2.setFrameShadow(QFrame.Raised)
        self.gridLayout_5 = QGridLayout(self.row_2)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.gridLayout_5.setHorizontalSpacing(0)
        self.gridLayout_5.setVerticalSpacing(10)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.dj_acc_client_id = QLineEdit(self.row_2)
        self.dj_acc_client_id.setObjectName(u"dj_acc_client_id")
        sizePolicy.setHeightForWidth(self.dj_acc_client_id.sizePolicy().hasHeightForWidth())
        self.dj_acc_client_id.setSizePolicy(sizePolicy)
        self.dj_acc_client_id.setMinimumSize(QSize(200, 30))
        self.dj_acc_client_id.setStyleSheet(u"QLineEdit {\n"
"	background-color: rgb(33, 37, 43);\n"
"	border-radius: 5px;\n"
"	border: 2px solid rgb(33, 37, 43);\n"
"	padding-left: 10px;\n"
"	selection-color: rgb(255, 255, 255);\n"
"	selection-background-color: rgb(255, 121, 198);\n"
"}\n"
"QLineEdit:hover {\n"
"	border: 2px solid rgb(64, 71, 88);\n"
"}\n"
"QLineEdit:focus {\n"
"	border: 2px solid rgb(91, 101, 124);\n"
"}")

        self.gridLayout_5.addWidget(self.dj_acc_client_id, 1, 0, 1, 1)

        self.labelVersion_7 = QLabel(self.row_2)
        self.labelVersion_7.setObjectName(u"labelVersion_7")
        sizePolicy.setHeightForWidth(self.labelVersion_7.sizePolicy().hasHeightForWidth())
        self.labelVersion_7.setSizePolicy(sizePolicy)
        self.labelVersion_7.setMinimumSize(QSize(0, 0))
        self.labelVersion_7.setStyleSheet(u"color: #f8f8f2; font: 14px")
        self.labelVersion_7.setLineWidth(1)
        self.labelVersion_7.setAlignment(Qt.AlignCenter)

        self.gridLayout_5.addWidget(self.labelVersion_7, 0, 0, 1, 1)


        self.verticalLayout_12.addWidget(self.row_2, 0, Qt.AlignVCenter)

        self.row_3 = QFrame(self.extraContent)
        self.row_3.setObjectName(u"row_3")
        self.row_3.setStyleSheet(u"border: 0;")
        self.row_3.setFrameShape(QFrame.StyledPanel)
        self.row_3.setFrameShadow(QFrame.Raised)
        self.gridLayout_3 = QGridLayout(self.row_3)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gridLayout_3.setHorizontalSpacing(0)
        self.gridLayout_3.setVerticalSpacing(10)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.labelVersion_5 = QLabel(self.row_3)
        self.labelVersion_5.setObjectName(u"labelVersion_5")
        sizePolicy.setHeightForWidth(self.labelVersion_5.sizePolicy().hasHeightForWidth())
        self.labelVersion_5.setSizePolicy(sizePolicy)
        self.labelVersion_5.setMinimumSize(QSize(0, 0))
        self.labelVersion_5.setFont(font3)
        self.labelVersion_5.setStyleSheet(u"color: #f8f8f2; font: 14px")
        self.labelVersion_5.setLineWidth(1)
        self.labelVersion_5.setAlignment(Qt.AlignCenter)

        self.gridLayout_3.addWidget(self.labelVersion_5, 0, 0, 1, 1)

        self.s3_role = QLineEdit(self.row_3)
        self.s3_role.setObjectName(u"s3_role")
        sizePolicy.setHeightForWidth(self.s3_role.sizePolicy().hasHeightForWidth())
        self.s3_role.setSizePolicy(sizePolicy)
        self.s3_role.setMinimumSize(QSize(200, 30))
        self.s3_role.setStyleSheet(u"QLineEdit {\n"
"	background-color: rgb(33, 37, 43);\n"
"	border-radius: 5px;\n"
"	border: 2px solid rgb(33, 37, 43);\n"
"	padding-left: 10px;\n"
"	selection-color: rgb(255, 255, 255);\n"
"	selection-background-color: rgb(255, 121, 198);\n"
"}\n"
"QLineEdit:hover {\n"
"	border: 2px solid rgb(64, 71, 88);\n"
"}\n"
"QLineEdit:focus {\n"
"	border: 2px solid rgb(91, 101, 124);\n"
"}")

        self.gridLayout_3.addWidget(self.s3_role, 1, 0, 1, 1)


        self.verticalLayout_12.addWidget(self.row_3, 0, Qt.AlignVCenter)

        self.row_4 = QFrame(self.extraContent)
        self.row_4.setObjectName(u"row_4")
        self.row_4.setStyleSheet(u"border: 0;")
        self.row_4.setFrameShape(QFrame.StyledPanel)
        self.row_4.setFrameShadow(QFrame.Raised)
        self.gridLayout_4 = QGridLayout(self.row_4)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.gridLayout_4.setHorizontalSpacing(0)
        self.gridLayout_4.setVerticalSpacing(10)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.s3_bucket = QLineEdit(self.row_4)
        self.s3_bucket.setObjectName(u"s3_bucket")
        sizePolicy.setHeightForWidth(self.s3_bucket.sizePolicy().hasHeightForWidth())
        self.s3_bucket.setSizePolicy(sizePolicy)
        self.s3_bucket.setMinimumSize(QSize(200, 30))
        self.s3_bucket.setStyleSheet(u"QLineEdit {\n"
"	background-color: rgb(33, 37, 43);\n"
"	border-radius: 5px;\n"
"	border: 2px solid rgb(33, 37, 43);\n"
"	padding-left: 10px;\n"
"	selection-color: rgb(255, 255, 255);\n"
"	selection-background-color: rgb(255, 121, 198);\n"
"}\n"
"QLineEdit:hover {\n"
"	border: 2px solid rgb(64, 71, 88);\n"
"}\n"
"QLineEdit:focus {\n"
"	border: 2px solid rgb(91, 101, 124);\n"
"}")

        self.gridLayout_4.addWidget(self.s3_bucket, 1, 0, 1, 1)

        self.labelVersion_6 = QLabel(self.row_4)
        self.labelVersion_6.setObjectName(u"labelVersion_6")
        sizePolicy.setHeightForWidth(self.labelVersion_6.sizePolicy().hasHeightForWidth())
        self.labelVersion_6.setSizePolicy(sizePolicy)
        self.labelVersion_6.setMinimumSize(QSize(0, 0))
        self.labelVersion_6.setStyleSheet(u"color: #f8f8f2; font: 14px")
        self.labelVersion_6.setLineWidth(1)
        self.labelVersion_6.setAlignment(Qt.AlignCenter)

        self.gridLayout_4.addWidget(self.labelVersion_6, 0, 0, 1, 1)


        self.verticalLayout_12.addWidget(self.row_4, 0, Qt.AlignVCenter)

        self.row_5 = QFrame(self.extraContent)
        self.row_5.setObjectName(u"row_5")
        self.row_5.setStyleSheet(u"border: 0;")
        self.row_5.setFrameShape(QFrame.StyledPanel)
        self.row_5.setFrameShadow(QFrame.Raised)
        self.gridLayout = QGridLayout(self.row_5)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.btn_save_config = QPushButton(self.row_5)
        self.btn_save_config.setObjectName(u"btn_save_config")
        sizePolicy2 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.btn_save_config.sizePolicy().hasHeightForWidth())
        self.btn_save_config.setSizePolicy(sizePolicy2)
        self.btn_save_config.setMinimumSize(QSize(150, 35))
        self.btn_save_config.setFont(font3)
        self.btn_save_config.setCursor(QCursor(Qt.PointingHandCursor))
        self.btn_save_config.setMouseTracking(True)
        self.btn_save_config.setStyleSheet(u"QPushButton {\n"
"	color: #f8f8f2;\n"
"	font: 14px;\n"
"	border: 2px solid rgb(52, 59, 72);\n"
"	border-radius: 5px;	\n"
"	background-color: rgb(52, 59, 72);\n"
"}\n"
"QPushButton:hover {\n"
"	background-color: rgb(57, 65, 80);\n"
"	border: 2px solid rgb(61, 70, 86);\n"
"}\n"
"QPushButton:pressed {	\n"
"	background-color: rgb(35, 40, 49);\n"
"	border: 2px solid rgb(43, 50, 61);\n"
"}")

        self.gridLayout.addWidget(self.btn_save_config, 0, 0, 1, 1)


        self.verticalLayout_12.addWidget(self.row_5, 0, Qt.AlignVCenter)


        self.extraColumLayout.addWidget(self.extraContent)


        self.horizontalLayout_6.addWidget(self.extraLeftBox)

        self.contentBox = QFrame(self.bgApp)
        self.contentBox.setObjectName(u"contentBox")
        self.contentBox.setFrameShape(QFrame.NoFrame)
        self.contentBox.setFrameShadow(QFrame.Raised)
        self.verticalLayout_2 = QVBoxLayout(self.contentBox)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.contentTopBg = QFrame(self.contentBox)
        self.contentTopBg.setObjectName(u"contentTopBg")
        self.contentTopBg.setMinimumSize(QSize(0, 50))
        self.contentTopBg.setMaximumSize(QSize(16777215, 50))
        self.contentTopBg.setFrameShape(QFrame.NoFrame)
        self.contentTopBg.setFrameShadow(QFrame.Raised)
        self.horizontalLayout = QHBoxLayout(self.contentTopBg)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 10, 0)
        self.leftBox = QFrame(self.contentTopBg)
        self.leftBox.setObjectName(u"leftBox")
        sizePolicy3 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.leftBox.sizePolicy().hasHeightForWidth())
        self.leftBox.setSizePolicy(sizePolicy3)
        self.leftBox.setFrameShape(QFrame.NoFrame)
        self.leftBox.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_3 = QHBoxLayout(self.leftBox)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.titleRightInfo = QLabel(self.leftBox)
        self.titleRightInfo.setObjectName(u"titleRightInfo")
        sizePolicy4 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        sizePolicy4.setHorizontalStretch(0)
        sizePolicy4.setVerticalStretch(0)
        sizePolicy4.setHeightForWidth(self.titleRightInfo.sizePolicy().hasHeightForWidth())
        self.titleRightInfo.setSizePolicy(sizePolicy4)
        self.titleRightInfo.setMaximumSize(QSize(16777215, 45))
        self.titleRightInfo.setFont(font)
        self.titleRightInfo.setStyleSheet(u"")
        self.titleRightInfo.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignVCenter)

        self.horizontalLayout_3.addWidget(self.titleRightInfo)


        self.horizontalLayout.addWidget(self.leftBox)

        self.rightButtons = QFrame(self.contentTopBg)
        self.rightButtons.setObjectName(u"rightButtons")
        self.rightButtons.setMinimumSize(QSize(0, 28))
        self.rightButtons.setFrameShape(QFrame.NoFrame)
        self.rightButtons.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_12 = QHBoxLayout(self.rightButtons)
        self.horizontalLayout_12.setSpacing(5)
        self.horizontalLayout_12.setObjectName(u"horizontalLayout_12")
        self.horizontalLayout_12.setContentsMargins(0, 0, 0, 0)
        self.rightButtons_2 = QFrame(self.rightButtons)
        self.rightButtons_2.setObjectName(u"rightButtons_2")
        self.rightButtons_2.setMinimumSize(QSize(0, 28))
        self.rightButtons_2.setFrameShape(QFrame.NoFrame)
        self.rightButtons_2.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_2 = QHBoxLayout(self.rightButtons_2)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.expTime = QLabel(self.rightButtons_2)
        self.expTime.setObjectName(u"expTime")
        sizePolicy5 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy5.setHorizontalStretch(0)
        sizePolicy5.setVerticalStretch(0)
        sizePolicy5.setHeightForWidth(self.expTime.sizePolicy().hasHeightForWidth())
        self.expTime.setSizePolicy(sizePolicy5)
        self.expTime.setMaximumSize(QSize(16777215, 45))
        self.expTime.setFont(font)
        self.expTime.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignVCenter)

        self.horizontalLayout_2.addWidget(self.expTime, 0, Qt.AlignRight|Qt.AlignVCenter)

        self.refreshTokenBtn = QPushButton(self.rightButtons_2)
        self.refreshTokenBtn.setObjectName(u"refreshTokenBtn")
        sizePolicy6 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        sizePolicy6.setHorizontalStretch(0)
        sizePolicy6.setVerticalStretch(0)
        sizePolicy6.setHeightForWidth(self.refreshTokenBtn.sizePolicy().hasHeightForWidth())
        self.refreshTokenBtn.setSizePolicy(sizePolicy6)
        self.refreshTokenBtn.setMinimumSize(QSize(75, 28))
        self.refreshTokenBtn.setMaximumSize(QSize(75, 28))
        self.refreshTokenBtn.setCursor(QCursor(Qt.PointingHandCursor))
        icon1 = QIcon()
        icon1.addFile(u":/icons/images/icons/cil-reload.png", QSize(), QIcon.Normal, QIcon.On)
        icon1.addFile(u":/icons/images/icons/cil-clock.png", QSize(), QIcon.Disabled, QIcon.On)
        self.refreshTokenBtn.setIcon(icon1)
        self.refreshTokenBtn.setIconSize(QSize(20, 20))

        self.horizontalLayout_2.addWidget(self.refreshTokenBtn)


        self.horizontalLayout_12.addWidget(self.rightButtons_2)

        self.themeSwitchBtn = QPushButton(self.rightButtons)
        self.themeSwitchBtn.setObjectName(u"themeSwitchBtn")
        self.themeSwitchBtn.setMinimumSize(QSize(28, 28))
        self.themeSwitchBtn.setMaximumSize(QSize(28, 28))
        self.themeSwitchBtn.setStyleSheet(u"color: white;")
        icon2 = QIcon()
        icon2.addFile(u":/icons/images/icons/cil-sun.png", QSize(), QIcon.Normal, QIcon.Off)
        self.themeSwitchBtn.setIcon(icon2)
        self.themeSwitchBtn.setIconSize(QSize(20, 20))

        self.horizontalLayout_12.addWidget(self.themeSwitchBtn)

        self.minimizeAppBtn = QPushButton(self.rightButtons)
        self.minimizeAppBtn.setObjectName(u"minimizeAppBtn")
        self.minimizeAppBtn.setMinimumSize(QSize(28, 28))
        self.minimizeAppBtn.setMaximumSize(QSize(28, 28))
        self.minimizeAppBtn.setCursor(QCursor(Qt.PointingHandCursor))
        icon3 = QIcon()
        icon3.addFile(u":/icons/images/icons/icon_minimize.png", QSize(), QIcon.Normal, QIcon.Off)
        self.minimizeAppBtn.setIcon(icon3)
        self.minimizeAppBtn.setIconSize(QSize(20, 20))

        self.horizontalLayout_12.addWidget(self.minimizeAppBtn)

        self.maximizeRestoreAppBtn = QPushButton(self.rightButtons)
        self.maximizeRestoreAppBtn.setObjectName(u"maximizeRestoreAppBtn")
        self.maximizeRestoreAppBtn.setMinimumSize(QSize(28, 28))
        self.maximizeRestoreAppBtn.setMaximumSize(QSize(28, 28))
        font4 = QFont()
        font4.setFamilies([u"Segoe UI"])
        font4.setPointSize(10)
        font4.setBold(False)
        font4.setItalic(False)
        font4.setStyleStrategy(QFont.PreferDefault)
        self.maximizeRestoreAppBtn.setFont(font4)
        self.maximizeRestoreAppBtn.setCursor(QCursor(Qt.PointingHandCursor))
        icon4 = QIcon()
        icon4.addFile(u":/icons/images/icons/icon_maximize.png", QSize(), QIcon.Normal, QIcon.Off)
        self.maximizeRestoreAppBtn.setIcon(icon4)
        self.maximizeRestoreAppBtn.setIconSize(QSize(20, 20))

        self.horizontalLayout_12.addWidget(self.maximizeRestoreAppBtn)

        self.closeAppBtn = QPushButton(self.rightButtons)
        self.closeAppBtn.setObjectName(u"closeAppBtn")
        self.closeAppBtn.setMinimumSize(QSize(28, 28))
        self.closeAppBtn.setMaximumSize(QSize(28, 28))
        self.closeAppBtn.setCursor(QCursor(Qt.PointingHandCursor))
        self.closeAppBtn.setIcon(icon)
        self.closeAppBtn.setIconSize(QSize(20, 20))

        self.horizontalLayout_12.addWidget(self.closeAppBtn)


        self.horizontalLayout.addWidget(self.rightButtons, 0, Qt.AlignRight)


        self.verticalLayout_2.addWidget(self.contentTopBg)

        self.contentBottom = QFrame(self.contentBox)
        self.contentBottom.setObjectName(u"contentBottom")
        self.contentBottom.setFrameShape(QFrame.NoFrame)
        self.contentBottom.setFrameShadow(QFrame.Raised)
        self.verticalLayout_6 = QVBoxLayout(self.contentBottom)
        self.verticalLayout_6.setSpacing(0)
        self.verticalLayout_6.setObjectName(u"verticalLayout_6")
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.content = QFrame(self.contentBottom)
        self.content.setObjectName(u"content")
        self.content.setFrameShape(QFrame.NoFrame)
        self.content.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_4 = QHBoxLayout(self.content)
        self.horizontalLayout_4.setSpacing(0)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.pagesContainer = QFrame(self.content)
        self.pagesContainer.setObjectName(u"pagesContainer")
        self.pagesContainer.setStyleSheet(u"")
        self.pagesContainer.setFrameShape(QFrame.NoFrame)
        self.pagesContainer.setFrameShadow(QFrame.Raised)
        self.verticalLayout_15 = QVBoxLayout(self.pagesContainer)
        self.verticalLayout_15.setSpacing(0)
        self.verticalLayout_15.setObjectName(u"verticalLayout_15")
        self.verticalLayout_15.setContentsMargins(10, 10, 10, 10)
        self.stackedWidget = QStackedWidget(self.pagesContainer)
        self.stackedWidget.setObjectName(u"stackedWidget")
        self.stackedWidget.setStyleSheet(u"background: transparent;")
        self.pageUpload = QWidget()
        self.pageUpload.setObjectName(u"pageUpload")
        self.verticalLayout_22 = QVBoxLayout(self.pageUpload)
        self.verticalLayout_22.setObjectName(u"verticalLayout_22")
        self.verticalLayout_22.setSizeConstraint(QLayout.SetDefaultConstraint)
        self.frame_3 = QFrame(self.pageUpload)
        self.frame_3.setObjectName(u"frame_3")
        self.frame_3.setFrameShape(QFrame.StyledPanel)
        self.frame_3.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_8 = QHBoxLayout(self.frame_3)
        self.horizontalLayout_8.setObjectName(u"horizontalLayout_8")
        self.frame_2 = QFrame(self.frame_3)
        self.frame_2.setObjectName(u"frame_2")
        sizePolicy3.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy3)
        self.frame_2.setMinimumSize(QSize(0, 0))
        self.frame_2.setFrameShape(QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QFrame.Raised)
        self.verticalLayout_7 = QVBoxLayout(self.frame_2)
        self.verticalLayout_7.setSpacing(2)
        self.verticalLayout_7.setObjectName(u"verticalLayout_7")
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.label = QLabel(self.frame_2)
        self.label.setObjectName(u"label")
        sizePolicy7 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy7.setHorizontalStretch(0)
        sizePolicy7.setVerticalStretch(0)
        sizePolicy7.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy7)
        self.label.setMinimumSize(QSize(0, 0))
        self.label.setStyleSheet(u"color: rgb(113, 126, 149); font: 14px")

        self.verticalLayout_7.addWidget(self.label)

        self.frame_7 = QFrame(self.frame_2)
        self.frame_7.setObjectName(u"frame_7")
        sizePolicy7.setHeightForWidth(self.frame_7.sizePolicy().hasHeightForWidth())
        self.frame_7.setSizePolicy(sizePolicy7)
        self.frame_7.setMinimumSize(QSize(0, 30))
        self.frame_7.setFrameShape(QFrame.StyledPanel)
        self.frame_7.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_10 = QHBoxLayout(self.frame_7)
        self.horizontalLayout_10.setSpacing(4)
        self.horizontalLayout_10.setObjectName(u"horizontalLayout_10")
        self.horizontalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.local_dir = QLineEdit(self.frame_7)
        self.local_dir.setObjectName(u"local_dir")
        sizePolicy1.setHeightForWidth(self.local_dir.sizePolicy().hasHeightForWidth())
        self.local_dir.setSizePolicy(sizePolicy1)
        self.local_dir.setMinimumSize(QSize(0, 30))
        self.local_dir.setStyleSheet(u"background-color: rgb(33, 37, 43);")
        self.local_dir.setReadOnly(True)

        self.horizontalLayout_10.addWidget(self.local_dir)

        self.btn_browse_dir = QPushButton(self.frame_7)
        self.btn_browse_dir.setObjectName(u"btn_browse_dir")
        sizePolicy8 = QSizePolicy(QSizePolicy.Maximum, QSizePolicy.Preferred)
        sizePolicy8.setHorizontalStretch(0)
        sizePolicy8.setVerticalStretch(0)
        sizePolicy8.setHeightForWidth(self.btn_browse_dir.sizePolicy().hasHeightForWidth())
        self.btn_browse_dir.setSizePolicy(sizePolicy8)
        self.btn_browse_dir.setMinimumSize(QSize(100, 0))
        self.btn_browse_dir.setCursor(QCursor(Qt.PointingHandCursor))
        self.btn_browse_dir.setStyleSheet(u"background-color: rgb(52, 59, 72); font: 12px")
        icon5 = QIcon()
        icon5.addFile(u":/icons/images/icons/cil-folder-open.png", QSize(), QIcon.Normal, QIcon.Off)
        self.btn_browse_dir.setIcon(icon5)

        self.horizontalLayout_10.addWidget(self.btn_browse_dir)


        self.verticalLayout_7.addWidget(self.frame_7)

        self.local_dir_tree = QTreeWidget(self.frame_2)
        __qtreewidgetitem = QTreeWidgetItem()
        __qtreewidgetitem.setText(0, u"1");
        self.local_dir_tree.setHeaderItem(__qtreewidgetitem)
        self.local_dir_tree.setObjectName(u"local_dir_tree")
        sizePolicy1.setHeightForWidth(self.local_dir_tree.sizePolicy().hasHeightForWidth())
        self.local_dir_tree.setSizePolicy(sizePolicy1)
        self.local_dir_tree.setMinimumSize(QSize(0, 0))
        self.local_dir_tree.setStyleSheet(u"font: 14px; background-color: rgb(33, 37, 43); color: #dddddd;border-radius: 5px;")
        self.local_dir_tree.setUniformRowHeights(True)
        self.local_dir_tree.setSortingEnabled(False)
        self.local_dir_tree.setAnimated(True)
        self.local_dir_tree.setHeaderHidden(True)
        self.local_dir_tree.setColumnCount(1)
        self.local_dir_tree.header().setVisible(False)

        self.verticalLayout_7.addWidget(self.local_dir_tree)


        self.horizontalLayout_8.addWidget(self.frame_2)

        self.frame_6 = QFrame(self.frame_3)
        self.frame_6.setObjectName(u"frame_6")
        self.frame_6.setFrameShape(QFrame.StyledPanel)
        self.frame_6.setFrameShadow(QFrame.Raised)
        self.verticalLayout_23 = QVBoxLayout(self.frame_6)
        self.verticalLayout_23.setObjectName(u"verticalLayout_23")
        self.verticalLayout_23.setContentsMargins(0, 0, 0, 0)
        self.btn_upload_files = QPushButton(self.frame_6)
        self.btn_upload_files.setObjectName(u"btn_upload_files")
        sizePolicy9 = QSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        sizePolicy9.setHorizontalStretch(0)
        sizePolicy9.setVerticalStretch(0)
        sizePolicy9.setHeightForWidth(self.btn_upload_files.sizePolicy().hasHeightForWidth())
        self.btn_upload_files.setSizePolicy(sizePolicy9)
        self.btn_upload_files.setMinimumSize(QSize(150, 30))
        self.btn_upload_files.setCursor(QCursor(Qt.PointingHandCursor))
        self.btn_upload_files.setStyleSheet(u"background-color: rgb(52, 59, 72); font: 14px")
        icon6 = QIcon()
        icon6.addFile(u":/icons/images/icons/cil-cloud-upload.png", QSize(), QIcon.Normal, QIcon.On)
        icon6.addFile(u":/icons/images/icons/cil-clock.png", QSize(), QIcon.Disabled, QIcon.On)
        self.btn_upload_files.setIcon(icon6)

        self.verticalLayout_23.addWidget(self.btn_upload_files)

        self.label_3 = QLabel(self.frame_6)
        self.label_3.setObjectName(u"label_3")
        sizePolicy7.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy7)
        self.label_3.setStyleSheet(u"color: #6272a4; font: 9px")
        self.label_3.setAlignment(Qt.AlignCenter)

        self.verticalLayout_23.addWidget(self.label_3)


        self.horizontalLayout_8.addWidget(self.frame_6, 0, Qt.AlignHCenter|Qt.AlignVCenter)

        self.frame = QFrame(self.frame_3)
        self.frame.setObjectName(u"frame")
        sizePolicy3.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy3)
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.verticalLayout_5 = QVBoxLayout(self.frame)
        self.verticalLayout_5.setSpacing(2)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.label_2 = QLabel(self.frame)
        self.label_2.setObjectName(u"label_2")
        sizePolicy7.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy7)
        self.label_2.setStyleSheet(u"color: rgb(113, 126, 149); font: 14px")

        self.verticalLayout_5.addWidget(self.label_2)

        self.frame_5 = QFrame(self.frame)
        self.frame_5.setObjectName(u"frame_5")
        sizePolicy7.setHeightForWidth(self.frame_5.sizePolicy().hasHeightForWidth())
        self.frame_5.setSizePolicy(sizePolicy7)
        self.frame_5.setMinimumSize(QSize(0, 30))
        self.frame_5.setFrameShape(QFrame.StyledPanel)
        self.frame_5.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_9 = QHBoxLayout(self.frame_5)
        self.horizontalLayout_9.setSpacing(4)
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.horizontalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.s3_dir = QLineEdit(self.frame_5)
        self.s3_dir.setObjectName(u"s3_dir")
        sizePolicy7.setHeightForWidth(self.s3_dir.sizePolicy().hasHeightForWidth())
        self.s3_dir.setSizePolicy(sizePolicy7)
        self.s3_dir.setMinimumSize(QSize(0, 30))
        self.s3_dir.setStyleSheet(u"background-color: rgb(33, 37, 43);")

        self.horizontalLayout_9.addWidget(self.s3_dir)

        self.btn_refresh_preview = QPushButton(self.frame_5)
        self.btn_refresh_preview.setObjectName(u"btn_refresh_preview")
        sizePolicy8.setHeightForWidth(self.btn_refresh_preview.sizePolicy().hasHeightForWidth())
        self.btn_refresh_preview.setSizePolicy(sizePolicy8)
        self.btn_refresh_preview.setMinimumSize(QSize(100, 0))
        self.btn_refresh_preview.setCursor(QCursor(Qt.PointingHandCursor))
        self.btn_refresh_preview.setStyleSheet(u"background-color: rgb(52, 59, 72); font: 12px")
        icon7 = QIcon()
        icon7.addFile(u":/icons/images/icons/cil-reload.png", QSize(), QIcon.Normal, QIcon.Off)
        self.btn_refresh_preview.setIcon(icon7)

        self.horizontalLayout_9.addWidget(self.btn_refresh_preview)


        self.verticalLayout_5.addWidget(self.frame_5)

        self.s3_bucket_dir = QTreeWidget(self.frame)
        __qtreewidgetitem1 = QTreeWidgetItem()
        __qtreewidgetitem1.setText(0, u"1");
        self.s3_bucket_dir.setHeaderItem(__qtreewidgetitem1)
        self.s3_bucket_dir.setObjectName(u"s3_bucket_dir")
        sizePolicy1.setHeightForWidth(self.s3_bucket_dir.sizePolicy().hasHeightForWidth())
        self.s3_bucket_dir.setSizePolicy(sizePolicy1)
        self.s3_bucket_dir.setMinimumSize(QSize(0, 0))
        self.s3_bucket_dir.setStyleSheet(u"font: 14px; background-color: rgb(33, 37, 43); color: #dddddd;border-radius: 5px;")
        self.s3_bucket_dir.setUniformRowHeights(True)
        self.s3_bucket_dir.setSortingEnabled(True)
        self.s3_bucket_dir.setAnimated(True)
        self.s3_bucket_dir.setHeaderHidden(True)
        self.s3_bucket_dir.setColumnCount(1)
        self.s3_bucket_dir.header().setVisible(False)

        self.verticalLayout_5.addWidget(self.s3_bucket_dir)


        self.horizontalLayout_8.addWidget(self.frame)


        self.verticalLayout_22.addWidget(self.frame_3)

        self.stackedWidget.addWidget(self.pageUpload)
        self.pageS3Browse = QWidget()
        self.pageS3Browse.setObjectName(u"pageS3Browse")
        self.pageS3Browse.setStyleSheet(u"b")
        self.horizontalLayout_7 = QHBoxLayout(self.pageS3Browse)
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.frame_4 = QFrame(self.pageS3Browse)
        self.frame_4.setObjectName(u"frame_4")
        sizePolicy3.setHeightForWidth(self.frame_4.sizePolicy().hasHeightForWidth())
        self.frame_4.setSizePolicy(sizePolicy3)
        self.frame_4.setFrameShape(QFrame.StyledPanel)
        self.frame_4.setFrameShadow(QFrame.Raised)
        self.verticalLayout_13 = QVBoxLayout(self.frame_4)
        self.verticalLayout_13.setSpacing(2)
        self.verticalLayout_13.setObjectName(u"verticalLayout_13")
        self.label_4 = QLabel(self.frame_4)
        self.label_4.setObjectName(u"label_4")
        sizePolicy7.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy7)
        self.label_4.setStyleSheet(u"color: rgb(113, 126, 149); font: 14px")

        self.verticalLayout_13.addWidget(self.label_4)

        self.frame_8 = QFrame(self.frame_4)
        self.frame_8.setObjectName(u"frame_8")
        sizePolicy7.setHeightForWidth(self.frame_8.sizePolicy().hasHeightForWidth())
        self.frame_8.setSizePolicy(sizePolicy7)
        self.frame_8.setMinimumSize(QSize(0, 30))
        self.frame_8.setFrameShape(QFrame.StyledPanel)
        self.frame_8.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_11 = QHBoxLayout(self.frame_8)
        self.horizontalLayout_11.setSpacing(4)
        self.horizontalLayout_11.setObjectName(u"horizontalLayout_11")
        self.horizontalLayout_11.setContentsMargins(0, 0, 0, 0)
        self.browse_s3_dir = QLineEdit(self.frame_8)
        self.browse_s3_dir.setObjectName(u"browse_s3_dir")
        sizePolicy7.setHeightForWidth(self.browse_s3_dir.sizePolicy().hasHeightForWidth())
        self.browse_s3_dir.setSizePolicy(sizePolicy7)
        self.browse_s3_dir.setMinimumSize(QSize(0, 30))
        self.browse_s3_dir.setStyleSheet(u"background-color: rgb(33, 37, 43);")

        self.horizontalLayout_11.addWidget(self.browse_s3_dir)

        self.btn_refresh_browser = QPushButton(self.frame_8)
        self.btn_refresh_browser.setObjectName(u"btn_refresh_browser")
        sizePolicy8.setHeightForWidth(self.btn_refresh_browser.sizePolicy().hasHeightForWidth())
        self.btn_refresh_browser.setSizePolicy(sizePolicy8)
        self.btn_refresh_browser.setMinimumSize(QSize(100, 0))
        self.btn_refresh_browser.setCursor(QCursor(Qt.PointingHandCursor))
        self.btn_refresh_browser.setStyleSheet(u"background-color: rgb(52, 59, 72); font: 12px")
        self.btn_refresh_browser.setIcon(icon7)

        self.horizontalLayout_11.addWidget(self.btn_refresh_browser)


        self.verticalLayout_13.addWidget(self.frame_8)

        self.browse_s3_bucket = QTreeWidget(self.frame_4)
        __qtreewidgetitem2 = QTreeWidgetItem()
        __qtreewidgetitem2.setText(0, u"1");
        self.browse_s3_bucket.setHeaderItem(__qtreewidgetitem2)
        self.browse_s3_bucket.setObjectName(u"browse_s3_bucket")
        sizePolicy1.setHeightForWidth(self.browse_s3_bucket.sizePolicy().hasHeightForWidth())
        self.browse_s3_bucket.setSizePolicy(sizePolicy1)
        self.browse_s3_bucket.setMinimumSize(QSize(0, 0))
        self.browse_s3_bucket.setStyleSheet(u"font: 14px; background-color: rgb(33, 37, 43); color: #dddddd;border-radius: 5px;")
        self.browse_s3_bucket.setUniformRowHeights(True)
        self.browse_s3_bucket.setSortingEnabled(True)
        self.browse_s3_bucket.setAnimated(True)
        self.browse_s3_bucket.setHeaderHidden(True)
        self.browse_s3_bucket.setColumnCount(1)
        self.browse_s3_bucket.header().setVisible(False)

        self.verticalLayout_13.addWidget(self.browse_s3_bucket)


        self.horizontalLayout_7.addWidget(self.frame_4)

        self.stackedWidget.addWidget(self.pageS3Browse)

        self.verticalLayout_15.addWidget(self.stackedWidget)


        self.horizontalLayout_4.addWidget(self.pagesContainer)


        self.verticalLayout_6.addWidget(self.content)

        self.bottomBar = QFrame(self.contentBottom)
        self.bottomBar.setObjectName(u"bottomBar")
        self.bottomBar.setMinimumSize(QSize(0, 22))
        self.bottomBar.setMaximumSize(QSize(16777215, 22))
        self.bottomBar.setFrameShape(QFrame.NoFrame)
        self.bottomBar.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_5 = QHBoxLayout(self.bottomBar)
        self.horizontalLayout_5.setSpacing(0)
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.horizontalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.toggleConsoleBtn = QPushButton(self.bottomBar)
        self.toggleConsoleBtn.setObjectName(u"toggleConsoleBtn")
        sizePolicy2.setHeightForWidth(self.toggleConsoleBtn.sizePolicy().hasHeightForWidth())
        self.toggleConsoleBtn.setSizePolicy(sizePolicy2)
        self.toggleConsoleBtn.setMinimumSize(QSize(20, 20))
        self.toggleConsoleBtn.setMaximumSize(QSize(20, 20))
        self.toggleConsoleBtn.setCursor(QCursor(Qt.PointingHandCursor))
        icon8 = QIcon()
        icon8.addFile(u":/icons/images/icons/cil-caret-right.png", QSize(), QIcon.Normal, QIcon.Off)
        self.toggleConsoleBtn.setIcon(icon8)
        self.toggleConsoleBtn.setIconSize(QSize(20, 20))

        self.horizontalLayout_5.addWidget(self.toggleConsoleBtn, 0, Qt.AlignLeft|Qt.AlignTop)

        self.upload_status = QTextEdit(self.bottomBar)
        self.upload_status.setObjectName(u"upload_status")
        self.upload_status.setFrameShape(QFrame.NoFrame)
        self.upload_status.setFrameShadow(QFrame.Plain)
        self.upload_status.setLineWidth(0)
        self.upload_status.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.upload_status.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.upload_status.setUndoRedoEnabled(False)
        self.upload_status.setReadOnly(True)
        self.upload_status.setTextInteractionFlags(Qt.NoTextInteraction)

        self.horizontalLayout_5.addWidget(self.upload_status)

        self.frame_size_grip = QFrame(self.bottomBar)
        self.frame_size_grip.setObjectName(u"frame_size_grip")
        sizePolicy2.setHeightForWidth(self.frame_size_grip.sizePolicy().hasHeightForWidth())
        self.frame_size_grip.setSizePolicy(sizePolicy2)
        self.frame_size_grip.setMinimumSize(QSize(20, 20))
        self.frame_size_grip.setMaximumSize(QSize(20, 16777215))
        self.frame_size_grip.setFrameShape(QFrame.NoFrame)
        self.frame_size_grip.setFrameShadow(QFrame.Raised)

        self.horizontalLayout_5.addWidget(self.frame_size_grip, 0, Qt.AlignRight|Qt.AlignBottom)


        self.verticalLayout_6.addWidget(self.bottomBar, 0, Qt.AlignTop)


        self.verticalLayout_2.addWidget(self.contentBottom)


        self.horizontalLayout_6.addWidget(self.contentBox)


        self.appMargins.addWidget(self.bgApp)

        MainWindow.setCentralWidget(self.styleSheet)

        self.retranslateUi(MainWindow)

        self.stackedWidget.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.titleLeftApp.setText(QCoreApplication.translate("MainWindow", u"Axon", None))
        self.titleLeftDescription.setText(QCoreApplication.translate("MainWindow", u"Upload files to S3", None))
        self.toggleButton.setText(QCoreApplication.translate("MainWindow", u"Hide", None))
        self.btn_upload.setText(QCoreApplication.translate("MainWindow", u"Upload", None))
        self.btn_bucket.setText(QCoreApplication.translate("MainWindow", u"S3 Bucket", None))
        self.toggleLeftBox.setText(QCoreApplication.translate("MainWindow", u"Config", None))
        self.extraLabel.setText(QCoreApplication.translate("MainWindow", u"DJ Sciops Config", None))
#if QT_CONFIG(tooltip)
        self.extraCloseColumnBtn.setToolTip(QCoreApplication.translate("MainWindow", u"Close left box", None))
#endif // QT_CONFIG(tooltip)
        self.extraCloseColumnBtn.setText("")
        self.aws_acc_id.setText("")
        self.aws_acc_id.setPlaceholderText("")
        self.labelVersion_4.setText(QCoreApplication.translate("MainWindow", u"AWS Account ID", None))
        self.dj_acc_client_id.setText("")
        self.dj_acc_client_id.setPlaceholderText("")
        self.labelVersion_7.setText(QCoreApplication.translate("MainWindow", u"DataJoint Account Client ID", None))
        self.labelVersion_5.setText(QCoreApplication.translate("MainWindow", u"S3 Role", None))
        self.s3_role.setText("")
        self.s3_role.setPlaceholderText("")
        self.s3_bucket.setText("")
        self.s3_bucket.setPlaceholderText("")
        self.labelVersion_6.setText(QCoreApplication.translate("MainWindow", u"S3 Bucket", None))
        self.btn_save_config.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.titleRightInfo.setText(QCoreApplication.translate("MainWindow", u"PyDracula APP - Theme with colors based on Dracula for Python.", None))
        self.expTime.setText(QCoreApplication.translate("MainWindow", u"Token Expiration Time: 00:00 AM", None))
        self.refreshTokenBtn.setText(QCoreApplication.translate("MainWindow", u"Token", None))
        self.themeSwitchBtn.setText("")
#if QT_CONFIG(tooltip)
        self.minimizeAppBtn.setToolTip(QCoreApplication.translate("MainWindow", u"Minimize", None))
#endif // QT_CONFIG(tooltip)
        self.minimizeAppBtn.setText("")
#if QT_CONFIG(tooltip)
        self.maximizeRestoreAppBtn.setToolTip(QCoreApplication.translate("MainWindow", u"Maximize", None))
#endif // QT_CONFIG(tooltip)
        self.maximizeRestoreAppBtn.setText("")
#if QT_CONFIG(tooltip)
        self.closeAppBtn.setToolTip(QCoreApplication.translate("MainWindow", u"Close", None))
#endif // QT_CONFIG(tooltip)
        self.closeAppBtn.setText("")
        self.label.setText(QCoreApplication.translate("MainWindow", u"Local Directory:", None))
        self.btn_browse_dir.setText(QCoreApplication.translate("MainWindow", u"Browse", None))
        self.btn_upload_files.setText(QCoreApplication.translate("MainWindow", u"Upload", None))
        self.label_3.setText(QCoreApplication.translate("MainWindow", u"*users will be authorized in a\n"
"new browser window", None))
        self.label_2.setText(QCoreApplication.translate("MainWindow", u"S3 Bucket Directory:", None))
        self.btn_refresh_preview.setText(QCoreApplication.translate("MainWindow", u"Preview", None))
        self.label_4.setText(QCoreApplication.translate("MainWindow", u"S3 Bucket Directory:", None))
        self.btn_refresh_browser.setText(QCoreApplication.translate("MainWindow", u"Load", None))
#if QT_CONFIG(tooltip)
        self.toggleConsoleBtn.setToolTip("")
#endif // QT_CONFIG(tooltip)
        self.toggleConsoleBtn.setText("")
    # retranslateUi

